@@ -36,11 +36,11 @@ static char rcsid[] =
  *	PVM console commands.
  *
 $Log: cmds.c,v $
- * Revision 1.25  1999/02/05  20:36:17  pvmsrc
- * More contributions from Edward Vigmond <vigmonde@IGB.UMontreal.CA>.
- * 	- better readline interface, with command completion & help.
- * (Spanker=kohl)
- *
+Revision 1.25  1999/02/05 20:36:17  pvmsrc
+More contributions from Edward Vigmond <vigmonde@IGB.UMontreal.CA>.
+	- better readline interface, with command completion & help.
+(Spanker=kohl)
+
  * Revision 1.24  1998/11/20  19:43:07  pvmsrc
  * Changes so that win32 will compile & build. Also, common
  * source for win32 & unix.
