@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: pvmdmimd.c,v 1.10 1997/10/01 15:36:06 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -86,6 +90,45 @@
  *			tp: task pointer.
  *
 $Log: pvmdmimd.c,v $
+Revision 1.10  1997/10/01 15:36:06  pvmsrc
+Removed unnecessary #include "fromlib.h" header.
+	- all consts now included in pvm3.h...
+	- header file eliminated.
+(Spanker=kohl)
+
+ * Revision 1.9  1997/08/27  20:18:58  pvmsrc
+ * Added blank args (0,0,0) in mpp_load() to make it call forkexec correctly.
+ * Protocol needs to be changed to allow taskers to access IBM poe directly.
+ * (Spanker=fagg)
+ *
+ * Revision 1.8  1997/07/09  13:54:58  pvmsrc
+ * Fixed Author Header.
+ *
+ * Revision 1.7  1997/06/02  13:48:38  pvmsrc
+ * Moved #include host.h above #include waitc.h.
+ * Removed old mesg.h include...  gone baby.
+ *
+ * Revision 1.6  1997/05/06  20:14:36  pvmsrc
+ * 	Catch stdout/stderr correctly and redirect to outtid,tag,ctx
+ *
+ * Revision 1.5  1997/05/05  20:08:56  pvmsrc
+ * 	Pass outctx and trcctx to mpp tasks.
+ *
+ * Revision 1.4  1997/05/02  13:52:50  pvmsrc
+ * 	Start up MPI jobs correctly and get them configured.
+ *
+ * Revision 1.3  1997/03/25  15:52:21  pvmsrc
+ * PVM patches from the base 3.3.10 to 3.3.11 versions where applicable.
+ * Originals by Bob Manchek. Altered by Graham Fagg where required.
+ * -IP enabled over switch
+ * -RMPOOL env can be used instead of host list
+ *
+ * Revision 1.2  1997/01/28  19:30:57  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.1  1996/09/23  23:15:09  pvmsrc
+ * Initial revision
+ *
  * Revision 1.3  1996/05/14  14:35:59  manchek
  * inc'd changes from chulho@kgn.ibm.com
  *
@@ -127,21 +170,18 @@ $Log: pvmdmimd.c,v $
 #define CINDEX(s,c) index(s,c)
 #endif
 
-#include "pvm3.h"
+#include <pvm3.h>
+#include <pvmproto.h>
 #include "global.h"
-#include "ddpro.h"
-#include "tdpro.h"
-#include "protoglarp.h"
-#include "pvmalloc.h"
-#include "mesg.h"
 #include "host.h"
+#include "waitc.h"
+#include "pvmalloc.h"
 #include "pkt.h"
 #include "task.h"
 #include "listmac.h"
 #include "pvmdmp.h"
 #include "pvmmimd.h"
 #include "bfunc.h"
-#include "fromlib.h"
 
 #define MPICOMM		"/usr/bin/poe"
 #define MPIOPT1		"-procs"
@@ -156,10 +196,9 @@ char *getenv();
 
 /* Global */
 
-extern int debugmask;			/* from pvmd.c */
+extern int pvmdebmask;			/* from pvmd.c */
 extern char **epaths;			/* from pvmd.c */
 extern int myhostpart;			/* from pvmd.c */
-extern int myndf;			/* from pvmd.c */
 extern int tidhmask;			/* from pvmd.c */
 extern int ourudpmtu;			/* from pvmd.c */
 extern struct htab *hosts;		/* from pvmd.c */
@@ -170,17 +209,20 @@ int tidnmask = TIDNODE;			/* mask for node field of tids */
 
 /* private */
 
-static char rcsid[] = "$Id: pvmdmimd.c,v 1.3 1996/05/14 14:35:59 manchek Exp $";
+static int myndf = 0;
 static struct nodeset *busynodes;	/* active nodes; ordered by proc type */
 static char pvmtxt[512];		/* scratch for error log */
 static int ptypemask;			/* mask; we use these bits of ptype in tids */
 static char nodefile[L_tmpnam];	/* tmp node file */
 static char **nodelist = 0;		/* default poe node list */
-static int partsize = 0;		/* total number of nodes allocated */
-static int hostfileused = TRUE;		/* Check if MP_HOSTFILE used */
-static char defaultpool[64]="1";	/* default MP_POOL if not set */
+static int partsize = 0;		/* number of nodes allocated */
+static int hostfileused = TRUE;         /* Check if MP_HOSTFILE used */
+static char defaultpool[64]="1";        /* default MP_POOL if not set */
 static char mpiadapter[]={MPIOPARG2};   /* default User-Space */
 
+static int sp2pvminfo[SIZEHINFO];
+
+
 
 void
 mpp_init(argc, argv)
@@ -292,7 +334,7 @@ sp2hostfile(first, count)
 		pvmlogperror(pvmtxt);
 		return (char *)0;
 	}
-	if (debugmask & PDMNODE) {
+	if (pvmdebmask & PDMNODE) {
 		sprintf(pvmtxt, "sp2hostfile() POE host file: %s\n", nodefile);
 		pvmlogerror(pvmtxt);
 	}
@@ -342,22 +384,22 @@ mpp_new(count, ptid)
 	for (sp2 = busynodes->n_link; sp2 != busynodes; sp2 = sp2->n_link)
 		if ((sp2->n_ptype & ptypemask) == (ptype & ptypemask))
 			break;
-	if (sp2 != busynodes || ptype == NUMPTYPE) {
-		for (ptype = 0; ptype < NUMPTYPE; ptype++) {
+	if (sp2 != busynodes || ptype == NPARTITIONS) {
+		for (ptype = 0; ptype < NPARTITIONS; ptype++) {
 			for (sp2 = busynodes->n_link; sp2 != busynodes; sp2 = sp2->n_link)
 				if ((sp2->n_ptype & ptypemask) == (ptype & ptypemask))
 					break;
 			if (sp2 == busynodes)
 				break;
 		}
-		if (ptype == NUMPTYPE) {
+		if (ptype == NPARTITIONS) {
 			pvmlogerror("mpp_new() out of ptypes: too many spawns\n");
 			return (struct nodeset *)0;
 		}
 	}
 
 done:
-	if (debugmask & PDMNODE) {
+	if (pvmdebmask & PDMNODE) {
 		sprintf(pvmtxt, "mpp_new() %d nodes %d ... ptype=%d ptid=%x\n",
 			count, last+1, ptype, ptid);
 		pvmlogerror(pvmtxt);
@@ -401,7 +443,7 @@ mpp_free(tp)
 	for (sp = busynodes->n_link; sp != busynodes; sp = sp->n_link) {
 		if ((sp->n_ptype & ptypemask) == ptype) {
 
-			if (debugmask & PDMNODE) {
+			if (pvmdebmask & PDMNODE) {
 				sprintf(pvmtxt, "mpp_free() t%x type=%ld alive=%d\n",
 					tid, sp->n_ptype, sp->n_alive);
 				pvmlogerror(pvmtxt);
@@ -437,16 +479,17 @@ mpp_free(tp)
 
 /* load executable onto the given set of nodes */
 int
-mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
-	int flags;              /* exec options */
+mpp_load( wxp )
+struct waitc_spawn *wxp;
+{
+	int flags = 0;              /* exec options */
 	char *name;             /* executable */
 	char **argv;            /* arg list (argv[-1] must be there) */
 	int count;				/* how many */
-	int tids[];				/* array to store new tids */
+	int *tids;				/* array to store new tids */
 	int ptid;				/* parent task ID */
 	int nenv;				/* length of environment */
 	char **envp;			/* environment strings */
-{
 	int ptypepart;			/* type field */
 	int j;
 	struct task *tp;
@@ -468,6 +511,18 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 
 	static char *nullep[] = { "", 0 };
 
+        /* -- initialize some variables from the waitc_spawn struct -- */
+
+        name = wxp->w_argv[0];
+        argv = wxp->w_argv;
+        count = wxp->w_veclen;
+        tids = wxp->w_vec;
+        ptid = wxp->w_ptid;
+        nenv = wxp->w_nenv;
+        envp = wxp->w_env;
+
+
+
 	eplist = CINDEX(name, '/') ? nullep : epaths;
 
 	for (ep = eplist; *ep; ep++) {
@@ -480,7 +535,7 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 		if (stat(path, &sb) == -1
 		|| ((sb.st_mode & S_IFMT) != S_IFREG)
 		|| !(sb.st_mode & S_IEXEC)) {
-			if (debugmask & PDMTASK) {
+			if (pvmdebmask & PDMTASK) {
 				sprintf(pvmtxt, "mpp_load() stat failed <%s>\n", path);
 				pvmlogerror(pvmtxt);
 			}
@@ -527,7 +582,10 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 */
 		if (flags & PvmTaskDebug)
 			av++;					/* pdbx name -procs # -euilib us */
-		if (err = forkexec(flags, av[0], av, 0, (char **)0, &tp))
+		/* if (err = forkexec(flags, av[0], av, 0, (char **)0, &tp))
+*/
+		if (err = forkexec(flags, av[0], av, 0, (char **)0, 0,
+0, 0, &tp))
 			goto done;
 		tp->t_ptid = ptid;
 		PVM_FREE(tp->t_a_out);
@@ -541,12 +599,21 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 		sprintf(htid, "PVMHTID=%d", tp->t_tid);
 		p = STRALLOC(htid);
 		pvmputenv(p);
+		sp2pvminfo[0] = TDPROTOCOL;
+		sp2pvminfo[1] = myhostpart + ptypepart;
+		sp2pvminfo[2] = ptid;
+		sp2pvminfo[3] = MAXFRAGSIZE;
+		sp2pvminfo[4] = myndf;
+		sp2pvminfo[5] = partsize;
+		sp2pvminfo[6] = wxp->w_outtid;
+		sp2pvminfo[7] = wxp->w_outtag;
+		sp2pvminfo[8] = wxp->w_outctx;
+		sp2pvminfo[9] = wxp->w_trctid;
+		sp2pvminfo[10] = wxp->w_trctag;
+		sp2pvminfo[11] = wxp->w_trcctx;
+
+
 /*
-		pvminfo[0] = TDPROTOCOL;
-		pvminfo[1] = myhostpart + ptypepart;
-		pvminfo[2] = ptid;
-		pvminfo[3] = MAXFRAGSIZE;
-		pvminfo[4] = myndf;
 		if (sockconn(sock, tp, pvminfo) == -1) {
 			err = PvmSysErr;
 			task_free(tp);
@@ -555,7 +622,7 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 */
 		/* XXX task may not be on same host; can't do auth with tmp file */
 		tp->t_flag |= TF_CONN;
-		if (debugmask & PDMTASK) {
+		if (pvmdebmask & PDMTASK) {
 			sprintf(pvmtxt, "mpp_load() %d type=%d ptid=%x t%x...\n",
 				count, sp->n_ptype, ptid, myhostpart + ptypepart);
 			pvmlogerror(pvmtxt);
@@ -573,10 +640,13 @@ mpp_load(flags, name, argv, count, tids, ptid, nenv, envp)
 			tp->t_txq = hosttxq;		/* node tasks share pvmhost's txq */
 			tp->t_out = hostout;        /* and stdout */
 			tp->t_pid = hostpid;		/* pvm_kill should kill pvmhost */
+			tp->t_outtid = wxp->w_outtid;  /* catch stdout/stderr  */
+			tp->t_outtag = wxp->w_outtag; 
+			tp->t_outctx = wxp->w_outctx; 
 		}
 		return 0;
 	}
-    if (debugmask & PDMTASK) {
+    if (pvmdebmask & PDMTASK) {
         sprintf(pvmtxt, "mpp_load() didn't find <%s>\n", name);
         pvmlogerror(pvmtxt);
     }
@@ -716,7 +786,7 @@ mpp_conn(tp, tp2)
 	int pvminfo[SIZEHINFO];			/* host info */
 	int ptypepart;					/* type field */
 
-	if (debugmask & PDMNODE) {
+	if (pvmdebmask & PDMNODE) {
 		sprintf(pvmtxt, "mpp_conn() pvmhost %x", tp2->t_tid);
 		pvmlogerror(pvmtxt);
 	}
@@ -733,13 +803,8 @@ mpp_conn(tp, tp2)
 		return -1;
 	}
 	ptypepart = (sp->n_ptype << (ffs(tidtmask) - 1)) | TIDONNODE;
-	pvminfo[0] = TDPROTOCOL;
-	pvminfo[1] = myhostpart + ptypepart;
-	pvminfo[2] = tp2->t_ptid;
-	pvminfo[3] = MAXFRAGSIZE;
-	pvminfo[4] = myndf;
-	if (write(tp2->t_sock, pvminfo, SIZEHINFO*sizeof(int)) 
-	!= SIZEHINFO*sizeof(int)) {
+	if (write(tp2->t_sock, sp2pvminfo, sizeof(sp2pvminfo)) 
+	!= sizeof(sp2pvminfo)) {
 		pvmlogperror("mpp_conn() write");
 		return -1;
 	}
@@ -812,7 +877,7 @@ sockconn(sock, tp, hinfo)
 		pvmlogperror("sockconn() accept");
 		return -1;
 	} else {
-		if (debugmask & (PDMPACKET|PDMTASK)) {
+		if (pvmdebmask & (PDMPACKET|PDMTASK)) {
 			sprintf(pvmtxt, "sockconn() accept from %s sock %d\n",
 				inadport_decimal(&tp->t_sad), tp->t_sock);
 			pvmlogerror(pvmtxt);
