@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: host.c,v 1.14 1998/11/20 20:03:57 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -32,7 +36,66 @@
  *	Host table functions.
  *
 $Log: host.c,v $
- * Revision 1.10  1996/05/13  20:25:48  manchek
+Revision 1.14  1998/11/20 20:03:57  pvmsrc
+Changes so that win32 will compile & build. Also, common
+Changes so that compiles & builds on NT. Also
+common source on win32 & unix.
+(Spanker=sscott)
+
+ * Revision 1.13  1998/06/26  12:31:07  pvmsrc
+ * Added IMA_FREEBSD to the ioctl() list of arches that have both
+ * 	SIOCGIFCONF and OSIOCGIFCONF defined...
+ * 	- need to force use of SIOCGIFCONF anyway for these bozos...
+ * (Spanker=kohl)
+ *
+ * Revision 1.12  1998/02/23  22:51:27  pvmsrc
+ * Added AIX4SP2 stuff.
+ * (Spanker=kohl)
+ *
+ * Revision 1.11  1997/12/05  16:23:37  pvmsrc
+ * Fixed setting of host table "ht_narch" field.
+ * 	- used by pvm_config() to return # of archs.
+ * 	- replaced old pvm_archcode() solution with new data signature
+ * 		stuff (hd_dsig field in host descriptor).
+ * (Spanker=kohl)
+ *
+ * Revision 1.10  1997/06/27  17:32:21  pvmsrc
+ * Updated for WIN32 header files & Authors.
+ *
+ * Revision 1.9  1997/06/24  20:39:16  pvmsrc
+ * Eliminated unnecessary global externs.
+ *
+ * Revision 1.8  1997/04/30  21:25:56  pvmsrc
+ * SGI Compiler Warning Cleanup.
+ *
+ * Revision 1.7  1997/04/09  14:38:20  pvmsrc
+ * PVM patches from the base 3.3.10 to 3.3.11 versions where applicable.
+ * Originals by Bob Manchek. Altered by Graham Fagg where required.
+ *
+ * Revision 1.6  1997/01/28  19:26:17  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.5  1996/10/25  14:04:44  pvmsrc
+ * changed ifdef around ioctl for AIX 3.x/4.x to use IMA_arch-class
+ *
+ * Revision 1.4  1996/10/25  13:57:17  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.3  1996/10/24  21:04:45  pvmsrc
+ * Moved #include of "global.h" down below other headers:
+ * 	- need to have all of the structures / types declared before
+ * 		the globals can be declared...
+ *
+ * Revision 1.2  1996/10/24  18:14:07  pvmsrc
+ * ifdefs to ioctl call parameter for AIX3.x/4.x difference
+ *
+ * Revision 1.1  1996/09/23  23:44:08  pvmsrc
+ * Initial revision
+ *
+ * Revision 1.10  1996/05/13  20:25:48  manchek/GEF
  * missing pvmlogerror in readhostfile
  *
  * Revision 1.9  1995/11/02  16:01:49  manchek
@@ -64,7 +127,30 @@ $Log: host.c,v $
  *
  */
 
+#ifndef WIN32
 #include <sys/param.h>
+#endif
+#ifdef NEEDMENDIAN
+#include <machine/endian.h>
+#endif
+#ifdef NEEDENDIAN
+#include <endian.h>
+#endif
+#ifdef NEEDSENDIAN
+#include <sys/endian.h>
+#endif
+
+#include <pvm3.h>
+#ifdef WIN32
+#include "pvmwin.h"
+#ifndef _XDR_HEADERS
+#define _XDR_HEADERS
+#include "..\xdr\types.h"
+#include "..\xdr\xdr.h"
+#endif
+#else
+#include <rpc/types.h>
+#include <rpc/xdr.h>
 #ifndef IMA_TITN
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -72,6 +158,7 @@ $Log: host.c,v $
 #include <bsd/sys/types.h>
 #include <sys/43ioctl.h>
 #endif
+
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -81,21 +168,23 @@ $Log: host.c,v $
 #include <net/if.h>
 #endif
 #include <netdb.h>
+#endif
+
 #include <stdio.h>
+
 #ifdef	SYSVSTR
 #include <string.h>
 #else
 #include <strings.h>
 #endif
 #include <ctype.h>
-#include "protoglarp.h"
-#include "global.h"
+#include <pvmproto.h>
 #include "pvmalloc.h"
 #include "host.h"
-#include "mesg.h"
+#include "pmsg.h"
 #include "pkt.h"
-#include "ddpro.h"
 #include "bfunc.h"
+#include "global.h"
 
 #ifndef	max
 #define	max(a,b)	((a)>(b)?(a):(b))
@@ -116,8 +205,7 @@ char *inadport_decimal();
  **           **
  ***************/
 
-extern int debugmask;				/* from pvmd.c */
-extern struct htab *hosts;			/* from pvmd.c */
+extern int pvmdebmask;				/* from pvmd.c */
 extern int tidhmask;				/* from pvmd.c */
 
 
@@ -126,9 +214,6 @@ extern int tidhmask;				/* from pvmd.c */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: host.c,v 1.10 1996/05/13 20:25:48 manchek Exp $";
-static char pvmtxt[1024];			/* scratch space for error messages */
-
 
 /*****************
  **  Interface  **
@@ -199,7 +284,7 @@ hd_free(hp)
 	if (hp->hd_rxq)
 		pk_free(hp->hd_rxq);
 	if (hp->hd_rxm)
-		mesg_unref(hp->hd_rxm);
+		pmsg_unref(hp->hd_rxm);
 	if (mcap = hp->hd_mcas) {
 		while (mcap->mc_link != mcap)
 			mca_free(mcap->mc_link);
@@ -226,22 +311,23 @@ hd_unref(hp)
 }
 
 
+void
 hd_dump(hp)
 	struct hostd *hp;
 {
 	int n;
 	struct pkt *pp, *pp2;
 
-	sprintf(pvmtxt,
-			" hd_dump() ref %d t%x n \"%s\" a \"%s\" ar \"%s\"\n",
+	pvmlogprintf(
+			" hd_dump() ref %d t 0x%x n \"%s\" a \"%s\" ar \"%s\" dsig 0x%x\n",
 			hp->hd_ref,
 			hp->hd_hostpart,
 			(hp->hd_name ? hp->hd_name : ""),
 			(hp->hd_aname ? hp->hd_aname : ""),
-			(hp->hd_arch ? hp->hd_arch : ""));
-	pvmlogerror(pvmtxt);
+			(hp->hd_arch ? hp->hd_arch : ""),
+			hp->hd_dsig);
 
-	sprintf(pvmtxt,
+	pvmlogprintf(
 			"           lo \"%s\" so \"%s\" dx \"%s\" ep \"%s\" bx \"%s\" wd \"%s\" sp %d\n",
 			(hp->hd_login ? hp->hd_login : ""),
 			(hp->hd_sopts ? hp->hd_sopts : ""),
@@ -250,25 +336,22 @@ hd_dump(hp)
 			(hp->hd_bpath ? hp->hd_bpath : ""),
 			(hp->hd_wdir ? hp->hd_wdir : ""),
 			hp->hd_speed);
-	pvmlogerror(pvmtxt);
 
 	n = 0;
 	if (pp2 = hp->hd_txq)
 		for (pp = pp2->pk_link; pp != pp2; pp = pp->pk_link)
 			n++;
-	sprintf(pvmtxt, "           sa %s mtu %d f 0x%x e %d txq %d\n",
+	pvmlogprintf("           sa %s mtu %d f 0x%x e %d txq %d\n",
 			inadport_decimal(&hp->hd_sad),
 			hp->hd_mtu,
 			hp->hd_flag,
 			hp->hd_err,
 			n);
-	pvmlogerror(pvmtxt);
 
-	sprintf(pvmtxt,
+	pvmlogprintf(
 			"           tx %d rx %d rtt %d.%06d\n",
 			hp->hd_txseq, hp->hd_rxseq,
 			hp->hd_rtt.tv_sec, hp->hd_rtt.tv_usec);
-	pvmlogerror(pvmtxt);
 }
 
 
@@ -371,6 +454,7 @@ ht_new(siz)
 *	Update ht_narch.
 */
 
+void
 ht_insert(htp, hp)
 	struct htab *htp;
 	struct hostd *hp;
@@ -379,6 +463,11 @@ ht_insert(htp, hp)
 	int i;
 	unsigned long mask = 0, tmpmask;
 
+	int *dsigs;
+	int ndsigs;
+	int found;
+	int d;
+
 	hh = (hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1);
 
 	/* extend ht_hosts[] if no room */
@@ -408,6 +497,7 @@ ht_insert(htp, hp)
 				PVM_FREE(hp2->hd_arch);
 			hp2->hd_arch = STRALLOC(hp->hd_arch);
 		}
+		hp2->hd_dsig = hp->hd_dsig;
 		hp2->hd_mtu = hp->hd_mtu;
 		hp2->hd_sad = hp->hd_sad;
 		hp2->hd_speed = hp->hd_speed;
@@ -422,17 +512,26 @@ ht_insert(htp, hp)
 
 	/* update number of arches */
 
+	dsigs = TALLOC( htp->ht_cnt, int, "dsigs");
+	ndsigs = 0;
+
 	htp->ht_narch = 0;
 	for (hh = htp->ht_last; hh > 0; hh--) {
 		hp = htp->ht_hosts[hh];
 		if (hp && hp->hd_arch) {
-			i = pvm_archcode(htp->ht_hosts[hh]->hd_arch);
-			tmpmask = (1 << i);
-			if (tmpmask & ~mask)
+			d = htp->ht_hosts[hh]->hd_dsig;
+			for ( i=0, found=0 ; i < ndsigs && !found ; i++ ) {
+				if ( dsigs[i] == d )
+					found++;
+			}
+			if ( !found ) {
+				dsigs[ ndsigs++ ] = d;
 				htp->ht_narch++;
-			mask |= tmpmask;
+			}
 		}
 	}
+
+	PVM_FREE( dsigs );
 }
 
 
@@ -441,6 +540,7 @@ ht_insert(htp, hp)
 *	Remove a host from a host table and unreference it.
 */
 
+void
 ht_delete(htp, hp)
 	struct htab *htp;
 	struct hostd *hp;
@@ -449,6 +549,11 @@ ht_delete(htp, hp)
 	int i;
 	unsigned long mask = 0, tmpmask;
 
+	int *dsigs;
+	int ndsigs;
+	int found;
+	int d;
+
 	hh = (hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1);
 	if (hh < 0 || hh > htp->ht_last || htp->ht_hosts[hh] != hp) {
 		pvmlogerror("ht_delete() host not in table\n");
@@ -462,31 +567,39 @@ ht_delete(htp, hp)
 
 	/* update number of arches */
 
+	dsigs = TALLOC( htp->ht_cnt, int, "dsigs");
+	ndsigs = 0;
+
 	htp->ht_narch = 0;
 	for (hh = htp->ht_last; hh > 0; hh--) {
-		if (!(hp = htp->ht_hosts[hh]))
-			continue;
-		if (hp->hd_arch) {
-			i = pvm_archcode(htp->ht_hosts[hh]->hd_arch);
-			tmpmask = (1 << i);
-			if (tmpmask & ~mask)
+		hp = htp->ht_hosts[hh];
+		if (hp && hp->hd_arch) {
+			d = htp->ht_hosts[hh]->hd_dsig;
+			for ( i=0, found=0 ; i < ndsigs && !found ; i++ ) {
+				if ( dsigs[i] == d )
+					found++;
+			}
+			if ( !found ) {
+				dsigs[ ndsigs++ ] = d;
 				htp->ht_narch++;
-			mask |= tmpmask;
+			}
 		}
 	}
+
+	PVM_FREE( dsigs );
 }
 
 
+void
 ht_dump(htp)
 	struct htab *htp;
 {
 	int hh;
 
-	sprintf(pvmtxt,
+	pvmlogprintf(
 "ht_dump() ser %d last %d cnt %d master %d cons %d local %d narch %d\n",
 			htp->ht_serial, htp->ht_last, htp->ht_cnt, htp->ht_master,
 			htp->ht_cons, htp->ht_local, htp->ht_narch);
-	pvmlogerror(pvmtxt);
 	for (hh = 0; hh <= htp->ht_last; hh++)
 		if (htp->ht_hosts[hh])
 			hd_dump(htp->ht_hosts[hh]);
@@ -568,8 +681,7 @@ parsehost(buf, hp)
 
 	ac = sizeof(av)/sizeof(av[0]);
 	if (acav(buf, &ac, av)) {
-		sprintf(pvmtxt, "parsehost(): line too long\n");
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("parsehost(): line too long\n");
 		goto bad;
 	}
 	if (!ac)
@@ -625,8 +737,7 @@ parsehost(buf, hp)
 			hp->hd_aname = STRALLOC(av[ac] + 3);
 			continue;
 		}
-		sprintf(pvmtxt, "parsehost(): unknown option \"%s\"\n", av[ac]);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("parsehost(): unknown option \"%s\"\n", av[ac]);
 		err++;
 	}
 	if (err)
@@ -673,14 +784,12 @@ readhostfile(fn)
 	*/
 
 	if (iflist(&my_in_addrs, &num_addrs) == -1 || num_addrs < 1) {
-		sprintf(pvmtxt, "readhostfile() iflist failed\n");
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("readhostfile() iflist failed\n");
 		goto bail;
 	}
 
 	if (!(ff = fopen(fn, "r"))) {
-		sprintf(pvmtxt, "readhostfile() %s: can't read\n", fn);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("readhostfile() %s: can't read\n", fn);
 		goto bail;
 	}
 
@@ -694,8 +803,7 @@ readhostfile(fn)
 			continue;
 
 		if (lnum > maxhostid) {
-			sprintf(pvmtxt, "readhostfile() %s: too many lines\n", fn);
-			pvmlogerror(pvmtxt);
+			pvmlogprintf("readhostfile() %s: too many lines\n", fn);
 			err++;
 			break;
 		}
@@ -710,8 +818,7 @@ readhostfile(fn)
 		}
 
 		if (parsehost(p, hp)) {
-			sprintf(pvmtxt, "readhostfile() %s %d: errors\n", fn, lnum);
-			pvmlogerror(pvmtxt);
+			pvmlogprintf("readhostfile() %s %d: errors\n", fn, lnum);
 			err++;
 			goto badline;
 		}
@@ -736,9 +843,8 @@ readhostfile(fn)
 	/* look up ip addr */
 
 		if (!(he = gethostbyname(hp->hd_aname ? hp->hd_aname : hp->hd_name))) {
-			sprintf(pvmtxt, "readhostfile() %s %d: %s: can't gethostbyname\n",
+			pvmlogprintf("readhostfile() %s %d: %s: can't gethostbyname\n",
 				fn, lnum, hp->hd_name);
-			pvmlogerror(pvmtxt);
 			err++;
 			goto badline;
 		}
@@ -764,8 +870,7 @@ badline:
 		hd_unref(hp);
 	}
 	if (err) {
-		sprintf(pvmtxt, "readhostfile() %s: %d errors in hostfile\n", fn, err);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("readhostfile() %s: %d errors in hostfile\n", fn, err);
 	}
 
 	if (defaults) {
@@ -817,7 +922,17 @@ iflist(alp, np)
 
 	sif.ifc_ifcu.ifcu_buf = buf;
 	sif.ifc_len = sizeof(buf);
-	if (ioctl(soc, SIOCGIFCONF, &sif) == -1) {
+	if (ioctl(soc,
+#ifdef OSIOCGIFCONF
+#if defined(IMA_RS6K) || defined(IMA_RS6KMP) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_FREEBSD)
+		SIOCGIFCONF
+#else
+		OSIOCGIFCONF
+#endif
+#else
+		SIOCGIFCONF
+#endif
+		, &sif) == -1) {
 		perror("ioctl");
 		goto bail;
 	}
@@ -839,18 +954,17 @@ iflist(alp, np)
 				iplist = TREALLOC(iplist, (nip + 10), struct in_addr);
 			iplist[nip++] =
 			((struct sockaddr_in*)(&reqp->ifr_ifru.ifru_addr))->sin_addr;
-			if (debugmask & PDMNET) {
+			if (pvmdebmask & PDMNET) {
 				long a;
 
 				a = ((struct sockaddr_in*)(&reqp->ifr_ifru.ifru_addr))->sin_addr.s_addr;
 				a = ntohl(a);
-				sprintf(pvmtxt, "iflist() %s %d.%d.%d.%d\n",
+				pvmlogprintf("iflist() %s %d.%d.%d.%d\n",
 						reqp->ifr_name,
 						0xff & (a >> 24),
 						0xff & (a >> 16),
 						0xff & (a >> 8),
 						0xff & a);
-				pvmlogerror(pvmtxt);
 			}
 		}
 	}
@@ -924,7 +1038,7 @@ acav(s, acp, av)
 {
 	register int ac;
 	register char *p = s;
-	register n = *acp;
+	register int n = *acp;
 
 	/* separate out words of command */
 
