@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: hoster.c,v 1.13 1999/01/28 18:56:21 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -32,10 +36,70 @@
  *	Slave pvmd starter.
  *
 $Log: hoster.c,v $
- * Revision 1.2  1996/05/17  21:33:50  manchek
- * missed a few bcopys
+Revision 1.13  1999/01/28 18:56:21  pvmsrc
+Added host add retries for alternate WIN32 pvmd command.
+	- added char *h_wincmd field to hst struct.
+	- in hoster() check for extra command string at end of each
+		host unpack (ignore if omitted), stick in h_wincmd ptr.
+	- in pl_startup() if default pvmd command fails,
+		and alternate WIN32 command is defined, reset h_cmd and
+		try phase1() again...
+	- in phase1(), if doing manual startup and an alternate WIN32
+		command is defined, echo that to the user too, so they can
+		try both before typing back the response...
+(Spanker=kohl)
+
+ * Revision 1.12  1998/11/20  20:03:58  pvmsrc
+ * Changes so that win32 will compile & build. Also, common
+ * Changes so that compiles & builds on NT. Also
+ * common source on win32 & unix.
+ * (Spanker=sscott)
  *
- * Revision 1.1  1996/05/13  20:26:12  manchek
+ * Revision 1.11  1997/12/01  19:20:34  pvmsrc
+ * Replaced #ifdef IMA_OS2 fd_set declarations:
+ * 	- new #ifdef FDSETNOTSTRUCT.
+ * 	- choose between "fd_set foo" and "struct fd_set foo"...
+ * (Spanker=kohl)
+ *
+ * Revision 1.10  1997/11/04  23:19:16  pvmsrc
+ * Cleaned up fd_set stuff (hopefully).
+ * (Spanker=kohl)
+ *
+ * Revision 1.9  1997/09/10  17:40:54  pvmsrc
+ * Oops...  bcopy() -> BCOPY()...
+ * (Spanker=kohl)
+ *
+ * Revision 1.8  1997/08/29  13:35:03  pvmsrc
+ * OS2 Port Submitted by Bohumir Horeni, horeni@login.cz.
+ * (Spanker=kohl)
+ *
+ * Revision 1.7  1997/06/25  22:08:46  pvmsrc
+ * Markus adds his frigging name to the author list of
+ * 	every file he ever looked at...
+ *
+ * Revision 1.6  1997/04/21  14:58:22  pvmsrc
+ * Changed #ifdefs that checked IMA_RS6K,IMA_SP2MPI & IMA_AIX46K
+ * 	to see if select.h was needed into single define NEEDSSELECTH.
+ * 	New archs need to set this in conf/
+ *
+ * Revision 1.5  1997/01/28  19:26:19  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.4  1996/10/25  13:57:19  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.3  1996/10/24  21:04:47  pvmsrc
+ * Moved #include of "global.h" down below other headers:
+ * 	- need to have all of the structures / types declared before
+ * 		the globals can be declared...
+ *
+ * Revision 1.2  1996/09/23  23:48:01  pvmsrc
+ * Initial Creation - original hoster.c.
+ *
+ * Revision 1.1  1996/09/23  23:44:09  pvmsrc
  * Initial revision
  *
  * Revision 1.17  1995/11/02  16:36:14  manchek
@@ -103,17 +167,33 @@ $Log: hoster.c,v $
 #ifdef NEEDSENDIAN
 #include <sys/endian.h>
 #endif
+
+#include <pvm3.h>
+#ifdef WIN32
+#ifndef _XDR_HEADERS
+#define _XDR_HEADERS
+#include "..\xdr\types.h"
+#include "..\xdr\xdr.h"
+#endif
+#else
+#include <rpc/types.h>
+#include <rpc/xdr.h>
+#include <sys/time.h>
+#include <sys/socket.h>
+#include <netinet/in.h>                                                  
+#include <netdb.h>
+#endif
+
 #ifdef IMA_TITN
 #include <bsd/sys/types.h>
 #else
 #include <sys/types.h>
 #endif
-#include <sys/time.h>
-#if defined(IMA_RS6K) || defined(IMA_SP2MPI)
+
+#ifdef NEEDSSELECTH
 #include <sys/select.h>
 #endif
-#include <sys/socket.h>
-#include <netinet/in.h>
+
 #include <fcntl.h>
 #include <stdio.h>
 #include <errno.h>
@@ -125,18 +205,15 @@ $Log: hoster.c,v $
 #include <strings.h>
 #define	CINDEX(s,c)	index(s,c)
 #endif
-#include <netdb.h>
 
-#include "global.h"
-#include "fromlib.h"
-#include "ddpro.h"
-#include "protoglarp.h"
+#include <pvmproto.h>
 #include "pvmalloc.h"
-#include "mesg.h"
+#include "pmsg.h"
 #include "host.h"
 #include "listmac.h"
 #include "tvdefs.h"
 #include "bfunc.h"
+#include "global.h"
 
 #ifndef	RSHCOMMAND
 #define	RSHCOMMAND	"/usr/ucb/rsh"
@@ -168,6 +245,7 @@ struct hst {
 #define	HST_PASSWORD	1		/* ask for a password */
 #define	HST_MANUAL		2		/* do manual startup */
 	char *h_cmd;
+	char *h_wincmd;				/* alternate WIN32 default pvmd cmd */
 	char *h_result;
 };
 
@@ -194,7 +272,7 @@ extern int errno;
 
 extern void pvmbailout();
 
-extern int debugmask;				/* from pvmd.c */
+extern int pvmdebmask;				/* from pvmd.c */
 extern char *username;				/* from pvmd.c */
 
 
@@ -203,17 +281,15 @@ extern char *username;				/* from pvmd.c */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: hoster.c,v 1.2 1996/05/17 21:33:50 manchek Exp $";
 static struct slot slots[RSHNPLL+2];	/* state var/context for each slot */
 static struct slot *slfree = 0;			/* free list of slots */
-static char pvmtxt[1024];				/* scratch for error log */
 
 
 int
 hoster(mp)
-	struct mesg *mp;
+	struct pmsg *mp;
 {
-	struct mesg *mp2;
+	struct pmsg *mp2;
 	int num;
 	int i;
 	struct hst **hostlist;
@@ -225,9 +301,8 @@ hoster(mp)
 	*/
 
 	upkint(mp, &num);
-	if (debugmask & PDMSTARTUP) {
-		sprintf(pvmtxt, "hoster() %d to start\n", num);
-		pvmlogerror(pvmtxt);
+	if (pvmdebmask & PDMSTARTUP) {
+		pvmlogprintf("hoster() %d to start\n", num);
 	}
 	if (num > 0) {
 		hostlist = TALLOC(num, struct hst *, "hsts");
@@ -243,12 +318,14 @@ hoster(mp)
 				pvmlogerror("hoster() bad message format\n");
 				pvmbailout(0);
 			}
-			if (debugmask & PDMSTARTUP) {
-				sprintf(pvmtxt, "%d. t%x %s so=\"%s\"\n", i,
+			/* Check for (possible) alternate WIN32 pvmd cmd */
+			if (upkstralloc(mp, &hp->h_wincmd))
+				hp->h_wincmd = 0;
+			if (pvmdebmask & PDMSTARTUP) {
+				pvmlogprintf("%d. t%x %s so=\"%s\"\n", i,
 						hp->h_tid,
 						hp->h_login,
 						hp->h_sopts);
-				pvmlogerror(pvmtxt);
 			}
 			if (p = CINDEX(hp->h_login, '@')) {
 				hp->h_name = STRALLOC(p + 1);
@@ -280,7 +357,7 @@ hoster(mp)
 
 	mp2 = mesg_new(0);
 	mp2->m_dst = mp->m_src;
-	mp2->m_cod = DM_STARTACK;
+	mp2->m_tag = DM_STARTACK;
 	mp2->m_wid = mp->m_wid;
 	pkint(mp2, num);
 	for (i = 0; i < num; i++) {
@@ -288,7 +365,7 @@ hoster(mp)
 		pkstr(mp2, hostlist[i]->h_result
 				? hostlist[i]->h_result : "PvmDSysErr");
 	}
-	if (debugmask & PDMSTARTUP)
+	if (pvmdebmask & PDMSTARTUP)
 		pvmlogerror("hoster() pvmd' sending back host table\n");
 	sendmessage(mp2);
 	work();		/* no return */
@@ -342,7 +419,11 @@ pl_startup(num, hostlist)
 	struct slot *sp, *sp2;
 	struct timeval tnow;
 	struct timeval tout;
+#ifdef FDSETNOTSTRUCT
+	fd_set rfds;						/* emx headers */
+#else
 	struct fd_set rfds;
+#endif
 	int nfds;
 	int i;
 	int n;
@@ -382,21 +463,26 @@ pl_startup(num, hostlist)
 			sp->s_hst = hp;
 			sp->s_len = 0;
 			sp->s_elen = 0;
-			if (debugmask & PDMSTARTUP) {
-				sprintf(pvmtxt, "pl_startup() trying %s\n", hp->h_name);
-				pvmlogerror(pvmtxt);
+			if (pvmdebmask & PDMSTARTUP) {
+				pvmlogprintf("pl_startup() trying %s\n", hp->h_name);
 			}
 			phase1(sp);
 			if (hp->h_result) {
 				/* error or fully started (manual startup) */
 
+				if (pvmdebmask & PDMSTARTUP) {
+					pvmlogprintf(
+					"pl_startup() got result \"%s\" for %s phase1()\n",
+						hp->h_result, hp->h_name);
+				}
+
 				LISTPUTBEFORE(slfree, sp, s_link, s_rlink);
 
 			} else {
 				/* partially started */
 
 				LISTPUTBEFORE(slact, sp, s_link, s_rlink);
-				gettimeofday(&sp->s_bail, (struct timezone*)0);
+				pvmgetclock(&sp->s_bail);
 				tout.tv_sec = RSHTIMEOUT;
 				tout.tv_usec = 0;
 				TVXADDY(&sp->s_bail, &sp->s_bail, &tout);
@@ -415,12 +501,11 @@ pl_startup(num, hostlist)
 		FD_ZERO(&rfds);
 		nfds = 0;
 		TVCLEAR(&tout);
-		gettimeofday(&tnow, (struct timezone*)0);
+		pvmgetclock(&tnow);
 		for (sp = slact->s_link; sp != slact; sp = sp->s_link) {
 			if (TVXLTY(&sp->s_bail, &tnow)) {
-				sprintf(pvmtxt, "pl_startup() giving up on host %s after %d secs\n",
+				pvmlogprintf("pl_startup() giving up on host %s after %d secs\n",
 						sp->s_hst->h_name, RSHTIMEOUT);
-				pvmlogerror(pvmtxt);
 				sp->s_hst->h_result = STRALLOC("PvmCantStart");
 				sp2 = sp->s_rlink;
 				close_slot(sp);
@@ -450,16 +535,15 @@ pl_startup(num, hostlist)
 		} else {
 			TVCLEAR(&tout);
 		}
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "pl_startup() select timeout is %d.%06d\n",
+		if (pvmdebmask & PDMSTARTUP) {
+			pvmlogprintf("pl_startup() select timeout is %d.%06d\n",
 					tout.tv_sec, tout.tv_usec);
-			pvmlogerror(pvmtxt);
 		}
 		if ((n = select(nfds,
 #ifdef	FDSETISINT
 				(int *)&rfds, (int *)0, (int *)0,
 #else
-				&rfds, (fd_set *)0, (fd_set *)0,
+				(fd_set *)&rfds, (fd_set *)0, (fd_set *)0,
 #endif
 				&tout)) == -1) {
 			if (errno != EINTR) {
@@ -467,9 +551,8 @@ pl_startup(num, hostlist)
 				pvmbailout(0);
 			}
 		}
-		if (debugmask & PDMSTARTUP) {
-			(void)sprintf(pvmtxt, "pl_startup() select returns %d\n", n);
-			pvmlogerror(pvmtxt);
+		if (pvmdebmask & PDMSTARTUP) {
+			pvmlogprintf("pl_startup() select returns %d\n", n);
 		}
 		if (n < 1) {
 			if (n == -1 && errno != EINTR) {
@@ -496,9 +579,8 @@ pl_startup(num, hostlist)
 					sp->s_ebuf[sp->s_elen] = 0;
 					while (p = CINDEX(sp->s_ebuf, '\n')) {
 						*p = 0;
-						sprintf(pvmtxt, "stderr@%s: %s\n",
+						pvmlogprintf("stderr@%s: %s\n",
 								sp->s_hst->h_name, sp->s_ebuf);
-						pvmlogerror(pvmtxt);
 						p++;
 						sp->s_elen -= p - sp->s_ebuf;
 						if (sp->s_elen > 0) {
@@ -507,17 +589,15 @@ pl_startup(num, hostlist)
 						}
 					}
 					if (sp->s_elen == sizeof(sp->s_ebuf) - 1) {
-						sprintf(pvmtxt, "stderr@%s: %s\n",
+						pvmlogprintf("stderr@%s: %s\n",
 								sp->s_hst->h_name, sp->s_ebuf);
-						pvmlogerror(pvmtxt);
 						sp->s_elen = 0;
 					}
 
 				} else {
 					if (sp->s_elen > 0) {
-						sprintf(pvmtxt, "stderr@%s: %s\n",
+						pvmlogprintf("stderr@%s: %s\n",
 								sp->s_hst->h_name, sp->s_ebuf);
-						pvmlogerror(pvmtxt);
 						sp->s_elen = 0;
 					}
 					(void)close(sp->s_efd);
@@ -538,18 +618,16 @@ pl_startup(num, hostlist)
 					while (p = CINDEX(sp->s_buf, '\n')) {
 						*p = 0;
 						if (!strncmp(sp->s_buf, "ddpro", 5)) {
-							if (debugmask & PDMSTARTUP) {
-								sprintf(pvmtxt, "stdout@%s: %s\n",
+							if (pvmdebmask & PDMSTARTUP) {
+								pvmlogprintf("stdout@%s: %s\n",
 										sp->s_hst->h_name, sp->s_buf);
-								pvmlogerror(pvmtxt);
 							}
 							sp->s_hst->h_result = STRALLOC(sp->s_buf);
 							break;
 
 						} else {
-							sprintf(pvmtxt, "stdout@%s: %s\n",
+							pvmlogprintf("stdout@%s: %s\n",
 									sp->s_hst->h_name, sp->s_buf);
-							pvmlogerror(pvmtxt);
 							p++;
 							sp->s_len -= p - sp->s_buf;
 							if (sp->s_len > 0) {
@@ -559,35 +637,54 @@ pl_startup(num, hostlist)
 						}
 					}
 					if (sp->s_len == sizeof(sp->s_buf) - 1) {
-						sprintf(pvmtxt, "stdout@%s: %s\n",
+						pvmlogprintf("stdout@%s: %s\n",
 								sp->s_hst->h_name, sp->s_buf);
-						pvmlogerror(pvmtxt);
 						sp->s_len = 0;
 					}
 
 				} else {
 					if (sp->s_len > 0) {
-						sprintf(pvmtxt, "stdout@%s: %s\n",
+						pvmlogprintf("stdout@%s: %s\n",
 								sp->s_hst->h_name, sp->s_buf);
-						pvmlogerror(pvmtxt);
 						sp->s_len = 0;
 					}
 					if (n) {
-						sprintf(pvmtxt, "stdout@%s",
+						pvmlogprintf("stdout@%s",
 								sp->s_hst->h_name);
-						pvmlogperror(pvmtxt);
 					} else {
-						sprintf(pvmtxt, "stdout@%s: EOF\n",
+						pvmlogprintf("stdout@%s: EOF\n",
 								sp->s_hst->h_name);
-						pvmlogerror(pvmtxt);
 					}
-					sp->s_hst->h_result = STRALLOC("PvmCantStart");
 					if (sp->s_elen > 0) {
-						sprintf(pvmtxt, "stderr@%s: %s\n",
+						pvmlogprintf("stderr@%s: %s\n",
 								sp->s_hst->h_name, sp->s_ebuf);
-						pvmlogerror(pvmtxt);
 						sp->s_elen = 0;
 					}
+
+					/* before failing, check for alternate WIN32 cmd */
+					if (sp->s_hst->h_wincmd) {
+						if (pvmdebmask & PDMSTARTUP) {
+							pvmlogprintf(
+								"pl_startup() re-trying %s as WIN32\n",
+								hp->h_name);
+						}
+						/* no need to free h_cmd, we're a pvmd'... */
+						sp->s_hst->h_cmd = sp->s_hst->h_wincmd;
+						sp->s_hst->h_wincmd = 0;
+
+						phase1(sp);
+
+						if (!(sp->s_hst->h_result)) {
+							/* partially started */
+							pvmgetclock(&sp->s_bail);
+							tout.tv_sec = RSHTIMEOUT;
+							tout.tv_usec = 0;
+							TVXADDY(&sp->s_bail, &sp->s_bail, &tout);
+						}
+
+					} else {
+						sp->s_hst->h_result = STRALLOC("PvmCantStart");
+					}
 				}
 				if (sp->s_hst->h_result) {
 					sp2 = sp->s_rlink;
@@ -619,7 +716,7 @@ phase1(sp)
 
 	if (!execport) {
 		if (!(se = getservbyname("exec", "tcp"))) {
-			sprintf(pvmtxt, "phase1() can't getservbyname(): %s\n", "exec");
+			pvmlogprintf("phase1() can't getservbyname(): %s\n", "exec");
 			pvmbailout(0);
 		}
 		execport = se->s_port;
@@ -638,15 +735,22 @@ phase1(sp)
 	if (hp->h_flag & HST_MANUAL) {
 		fprintf(stderr, "*** Manual startup ***\n");
 		fprintf(stderr, "Login to \"%s\" and type:\n", hn);
-		fprintf(stderr, "%s\n", hp->h_cmd);
+
+		if (hp->h_wincmd) {
+			fprintf(stderr, "%s\n\n", hp->h_cmd);
+			fprintf(stderr, "or, if that command fails, " );
+			fprintf(stderr, "for WIN32 hosts try:\n");
+			fprintf(stderr, "%s\n\n", hp->h_wincmd);
+		}
+		else
+			fprintf(stderr, "%s\n", hp->h_cmd);
 
 	/* get version */
 
 		fprintf(stderr, "Type response: ");
 		fflush(stderr);
 		if (!(fgets(buf, sizeof(buf), stdin))) {
-			sprintf(pvmtxt, "host %s read error\n", hn);
-			pvmlogerror(pvmtxt);
+			pvmlogprintf("host %s read error\n", hn);
 			goto oops;
 		}
 		p = buf + strlen(buf) - 1;
@@ -666,9 +770,8 @@ phase1(sp)
 		int wpfd[2], rpfd[2], epfd[2];
 		int i;
 
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "phase1() trying rsh to %s\n", hn);
-			pvmlogerror(pvmtxt);
+		if (pvmdebmask & PDMSTARTUP) {
+			pvmlogprintf("phase1() trying rsh to %s\n", hn);
 		}
 
 	/* fork an rsh to startup the slave pvmd */
@@ -687,10 +790,9 @@ phase1(sp)
 		}
 #endif
 
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "phase1() pipes: %d %d %d %d %d %d\n",
+		if (pvmdebmask & PDMSTARTUP) {
+			pvmlogprintf("phase1() pipes: %d %d %d %d %d %d\n",
 					wpfd[0], wpfd[1], rpfd[0], rpfd[1], epfd[0], epfd[1]);
-			pvmlogerror(pvmtxt);
 		}
 
 		if ((pid = fork()) == -1) {
@@ -712,7 +814,7 @@ phase1(sp)
 			}
 			av[ac++] = hp->h_cmd;
 			av[ac++] = 0;
-			if (debugmask & PDMSTARTUP) {
+			if (pvmdebmask & PDMSTARTUP) {
 				for (ac = 0; av[ac]; ac++)
 					fprintf(stderr, "av[%d]=\"%s\" ", ac, av[ac]);
 				fputc('\n', stderr);
@@ -732,20 +834,17 @@ phase1(sp)
 	} else {		/* use rexec to start */
 
 #ifdef NOREXEC
-		sprintf(pvmtxt, "slconfg() PVM was built without rexec support, can't take passwords\n");
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("slconfg() PVM was built without rexec support, can't take passwords\n");
 		goto oops;
 #else
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "phase1() rexec \"%s\"\n", hp->h_cmd);
-			pvmlogerror(pvmtxt);
+		if (pvmdebmask & PDMSTARTUP) {
+			pvmlogprintf("phase1() rexec \"%s\"\n", hp->h_cmd);
 		}
 		if ((sp->s_wfd = sp->s_rfd = rexec(&hn, execport,
 				(hp->h_login ? hp->h_login : username),
 				(char*)0, hp->h_cmd, &sp->s_efd))
 		== -1) {
-			sprintf(pvmtxt, "phase1() rexec failed for host %s\n", hn);
-			pvmlogerror(pvmtxt);
+			pvmlogprintf("phase1() rexec failed for host %s\n", hn);
 			goto oops;
 		}
 #endif
