@@ -36,12 +36,12 @@ static char rcsid[] =
  *	Child task output collection.
  *
 $Log: lpvmcat.c,v $
- * Revision 1.10  1998/11/20  20:04:02  pvmsrc
- * Changes so that win32 will compile & build. Also, common
- * Changes so that compiles & builds on NT. Also
- * common source on win32 & unix.
- * (Spanker=sscott)
- *
+Revision 1.10  1998/11/20 20:04:02  pvmsrc
+Changes so that win32 will compile & build. Also, common
+Changes so that compiles & builds on NT. Also
+common source on win32 & unix.
+(Spanker=sscott)
+
  * Revision 1.9  1998/01/12  21:13:24  pvmsrc
  * Replaced inline constants with new task output op defines.
  * 	- TO_NEW == -2.
