@@ -36,11 +36,11 @@ static char rcsid[] =
  *	Libpvm core for MPP environment.
  *
 $Log: lpvmshmem.c,v $
- * Revision 1.41  1999/03/15  19:06:03  pvmsrc
- * Replaced sleep() calls with pvmsleep().
- * 	- Unix / Win32, C / Fortran compat...
- * (Spanker=kohl)
- *
+Revision 1.41  1999/03/15 19:06:03  pvmsrc
+Replaced sleep() calls with pvmsleep().
+	- Unix / Win32, C / Fortran compat...
+(Spanker=kohl)
+
  * Revision 1.40  1999/02/20  14:17:57  pvmsrc
  * Another try at the SGI compiler warning...  :-]
  * (Spanker=kohl)
