@@ -36,12 +36,12 @@ static char rcsid[] =
  *	Libpvm and pvmd message descriptors
  *
 $Log: pmsg.c,v $
- * Revision 1.12  1998/11/20  20:06:34  pvmsrc
- * Changes so that win32 will compile & build. Also, common
- * Changes so that compiles & builds on NT. Also
- * common source on win32 & unix.
- * (spanker=sscott)
- *
+Revision 1.12  1998/11/20 20:06:34  pvmsrc
+Changes so that win32 will compile & build. Also, common
+Changes so that compiles & builds on NT. Also
+common source on win32 & unix.
+(spanker=sscott)
+
  * Revision 1.11  1998/03/10  20:32:31  pvmsrc
  * Fixed tracing bogusness in struct timeval on 64-bit arches...
  * 	- need to cast to (int) before packing, else could crop off usecs...
