@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: pvmcruft.c,v 1.24 1999/03/16 16:29:56 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -32,6 +36,114 @@
  *	Missing links and other wonk.
  *
 $Log: pvmcruft.c,v $
+Revision 1.24  1999/03/16 16:29:56  pvmsrc
+Oops...  Fixed typing problem for WIN32 in pvmsleep()...
+(Spanker=kohl)
+
+ * Revision 1.23  1999/03/15  19:06:32  pvmsrc
+ * Added new pvmsleep() routine.
+ * 	- Unix / Win32, C / Fortran compat...
+ * 	- invokes Sleep() on WIN32 (with secs * 1000 :-)~
+ * (Spanker=kohl)
+ *
+ * Revision 1.22  1999/03/05  17:21:15  pvmsrc
+ * improved to work with registry/environment on NT/win95-98
+ * and devstudio 5/6
+ * (Spanker=sscott)
+ *
+ * Revision 1.21  1999/02/09  23:06:53  pvmsrc
+ * Cleaned up temp file stuff (mainly for WIN32).
+ * 	- extracted new pvmgettmp() routine for returning tmp directory.
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  1999/02/09  20:10:10  pvmsrc
+ * Cleaned up pvmgetroot().
+ * 	- some arch cases not properly executed.
+ * 	- for WIN32, if read_pvmregistry() fails, give the env var a shot.
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  1998/10/02  15:44:04  pvmsrc
+ * Single source code merge of Win32 and Unix code.
+ * (Spanker=sscott)
+ *
+ * Revision 1.18  1998/09/23  15:23:42  pvmsrc
+ * 	changes to use WIN32 registry as per Markus.
+ * 	ifdef in pvmd.c::colonsep() to include WIN32 and OS2
+ * (Spanker=phil)
+ *
+ * Revision 1.17  1997/12/01  19:20:48  pvmsrc
+ * Replaced #ifdef IMA_OS2 fd_set declarations:
+ * 	- new #ifdef FDSETNOTSTRUCT.
+ * 	- choose between "fd_set foo" and "struct fd_set foo"...
+ * (Spanker=kohl)
+ *
+ * Revision 1.16  1997/11/04  23:19:25  pvmsrc
+ * Cleaned up fd_set stuff (hopefully).
+ * (Spanker=kohl)
+ *
+ * Revision 1.15  1997/06/27  19:19:34  pvmsrc
+ * Integrated WIN32 changes.
+ *
+ * Revision 1.14  1997/05/27  14:47:04  pvmsrc
+ * New message type added: TM_SHMCONN.
+ *
+ * Revision 1.13  1997/04/30  21:26:10  pvmsrc
+ * SGI Compiler Warning Cleanup.
+ *
+ * Revision 1.12  1997/04/21  14:58:23  pvmsrc
+ * Changed #ifdefs that checked IMA_RS6K,IMA_SP2MPI & IMA_AIX46K
+ * 	to see if select.h was needed into single define NEEDSSELECTH.
+ * 	New archs need to set this in conf/
+ *
+ * Revision 1.11  1997/04/08  19:41:41  pvmsrc
+ * Added "DM_RESET", "DM_RESETACK" to taglist2...
+ *
+ * Revision 1.10  1997/04/08  17:23:33  pvmsrc
+ * Damn, Damn, Damn...
+ * 	- pvmreset() must be in library side only, pvmds use different
+ * 		message buffer mgt, packing and sending routines...  :-Q
+ *
+ * Revision 1.9  1997/04/08  17:08:17  pvmsrc
+ * Oops...  need to pass in caller's tid, lest they be reset, too...
+ *
+ * Revision 1.8  1997/04/07  20:51:18  pvmsrc
+ * Added pvmreset() routine.
+ * 	- for use by "pvm" console / XPVM, etc.
+ * 	- wipe out tasks, and tell master pvmd to clean up mboxes.
+ *
+ * Revision 1.7  1997/03/06  21:50:23  pvmsrc
+ * Yanked out #includes for <netinet/in.h> and <netinet/tcp.h>.
+ * 	- dups with lpvm.h #includes...
+ *
+ * Revision 1.6  1997/03/06  21:09:47  pvmsrc
+ * 	use dclock() for pvmgetclock()
+ *
+ * Revision 1.5  1997/01/28  19:27:00  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.4  1996/10/25  13:57:57  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.3  1996/10/24  21:30:39  pvmsrc
+ * Moved #include "global.h" to below other #includes for typing.
+ * Added #include of new "lpvm.h" & replaced explicit externs.
+ * Added new trace control constants:
+ * 	- TM_TRACER, for pvm_reg_tracer() messages.
+ * 	- TC_SETTRACE, TC_SETTRCBUF, TC_SETTRCOPT, for on-the-fly setopt
+ * 		control message / handlers.
+ *
+ * Revision 1.2  1996/09/27  13:17:09  pvmsrc
+ * Added special #include of <sys/select.h> for old RS6K (AIX 3.2):
+ * 	- fd_set declarations not found in <sys/time.h> on those systems,
+ * 		or evidently in SP2MPI, need <sys/select.h>.
+ * 	- similar fixes already existed in hoster.c, lpvm.c and pvmd.c.
+ *
+ * Revision 1.1  1996/09/23  23:44:29  pvmsrc
+ * Initial revision
+ *
  * Revision 1.10  1995/11/02  16:21:34  manchek
  * made pvmhdump write to pvm log.
  * new switch NEEDSFFS for ffs.
@@ -72,10 +184,16 @@ $Log: pvmcruft.c,v $
 #else
 #include <sys/types.h>
 #endif
+#ifndef WIN32
 #include <sys/time.h>
-#include <sys/stat.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
+#else 
+#include <time.h>
+#endif
+#ifdef NEEDSSELECTH
+#include <sys/select.h>
+#endif
+#include <sys/stat.h>
 #include <ctype.h>
 #ifdef	SYSVSTR
 #include <string.h>
@@ -84,12 +202,11 @@ $Log: pvmcruft.c,v $
 #include <strings.h>
 #define	CINDEX(s,c)	index(s,c)
 #endif
-#include "global.h"
-#include "protoglarp.h"
+#include <pvm3.h>
+#include <pvmproto.h>
 #include "pvmalloc.h"
-#include "tdpro.h"
-#include "ddpro.h"
-#include <pvmsdpro.h>
+#include "lpvm.h"
+#include "global.h"
 
 
 /***************
@@ -97,18 +214,18 @@ $Log: pvmcruft.c,v $
  **           **
  ***************/
 
+char *pvmgettmp();
 char *getenv();
 
-extern int pvm_useruid;				/* from pvmd.c or lpvm.c */
-
+#ifdef WIN32
+extern char *username;
+#endif
 
 /***************
  **  Private  **
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmcruft.c,v 1.10 1995/11/02 16:21:34 manchek Exp $";
-
 
 int
 pvmput32(p, n)
@@ -193,15 +310,23 @@ getdtablesize()
 
 #else /*IMA_SX3*/
 
+#ifdef WIN32
+int
+getdtablesize()
+{
+	return 64;
+}
+#else
 #include <unistd.h>
-
 int
 getdtablesize()
 {
 	return sysconf(_SC_OPEN_MAX);
 }
+#endif
 
 #endif /*IMA_SX3*/
+
 #endif	/*IMA_TITN*/
 
 #endif /*NOGETDTBLSIZ*/
@@ -224,8 +349,9 @@ ffs(x)
 
 #endif
 
-
+#ifndef IMA_WIN32_WATCOM
 extern char **environ;
+#endif
 
 /*	pvmputenv()
 *
@@ -408,6 +534,8 @@ pvmxtoi(p)
 	int i = 0;
 	char c;
 
+	if (p[0] == '0' && (p[1] == 'x' || p[1] == 'X'))
+		p += 2;
 	while (isxdigit(c = *p++)) {
 		i = (i << 4) + c - (isdigit(c) ? '0' : (isupper(c) ? 'A' : 'a') - 10);
 	}
@@ -415,6 +543,33 @@ pvmxtoi(p)
 }
 
 
+/*	pvmstrtoi()
+*
+*	Convert string in decimal, hex or octal to integer.
+*/
+
+int
+pvmstrtoi(p)
+	char *p;
+{
+	int i = 0;
+	char c;
+
+	if (*p == '0') {
+		p++;
+		if (*p == 'x' || *p == 'X')
+			while (isxdigit(c = *++p))
+				i = (i << 4) + c
+						- (isdigit(c) ? '0' : (isupper(c) ? 'A' : 'a') - 10);
+		else
+			while (c = *p++, c >= '0' && c <= '7')
+				i = (i << 3) + c - '0';
+		return i;
+	} else
+		return (int)atoi(p);
+}
+
+
 #ifndef	I860_NODE
 
 /*	hex_inadport()
@@ -422,6 +577,7 @@ pvmxtoi(p)
 *	Takes a string of format 00000000:0000 and returns a sockaddr_in.
 */
 
+void
 hex_inadport(s, sad)
 	char *s;
 	struct sockaddr_in *sad;
@@ -481,11 +637,17 @@ pvmtmpnam(buf)
 	char *buf;
 {
 	static int n = 0;
-	static char scratch[32];
+	static char scratch[255];
+
+	char *pvmtmp;
 
 	if (!buf)
 		buf = scratch;
-	sprintf(buf, "/tmp/tmp%06d.%d", getpid(), n++ % 10000);
+
+	pvmtmp = pvmgettmp();
+
+	sprintf(buf, "%s/tmp%06d.%d", pvmtmp, getpid(), n++ % 10000);
+
 	return buf;
 }
 #endif	/*NOTMPNAM*/
@@ -541,7 +703,7 @@ char *
 pvmgethome()
 {
 	static char *hd = 0;
-
+#ifndef WIN32 /* which registry should we use otherwise ? */
 	if (!hd) {
 		if (hd = getenv("HOME")) {
 			hd = STRALLOC(hd);
@@ -551,6 +713,9 @@ pvmgethome()
 			hd = "/";
 		}
 	}
+#else
+	hd="\\";
+#endif
 	return hd;
 }
 
@@ -565,26 +730,65 @@ pvmgetroot()
 {
 	static char *rd = 0;
 
-	if (!rd && !(rd = getenv("PVM_ROOT"))) {
-		pvmlogerror("PVM_ROOT environment variable not set.\n");
-#ifdef IMA_CSPP
-		{
-			struct stat buf;
+	if (!rd) {
 
-			rd = STRALLOC("/usr/convex/pvm");
-			if (stat(rd, &buf) == -1) {
-				pvmlogperror("Unable to default PVM_ROOT to /usr/convex/pvm");
-				pvmbailout(0);
-				exit(1);		/* the other meaning of bail out */
-			}
-			pvmputenv("PVM_ROOT=/usr/convex/pvm");
-			pvmlogerror("Defaulting PVM_ROOT to /usr/convex/pvm");
+#ifdef IMA_CSPP
+
+		struct stat buf;
+
+		rd = STRALLOC("/usr/convex/pvm");
+		if (stat(rd, &buf) == -1) {
+			pvmlogperror(
+				"Unable to default PVM_ROOT to /usr/convex/pvm");
+			pvmbailout(0);
+			exit(1);		/* the other meaning of bail out */
+		}
+		pvmputenv("PVM_ROOT=/usr/convex/pvm");
+		pvmlogerror("Defaulting PVM_ROOT to /usr/convex/pvm");
+
+#else
+
+		rd = getenv("PVM_ROOT");
+
+#endif
+/* IMA_CSPP */
+
+		if (!rd) {
+			pvmlogerror("PVM_ROOT environment variable not set.\n");
+			pvmbailout(0);
+			exit(1);		/* the other meaning of bail out */
+		}
+	}
+
+	return rd;
+}
+
+
+/*	pvmgettmp()
+*
+*	Return absolute path of PVM temporary directory.
+*/
+
+char *
+pvmgettmp()
+{
+	static char *td = 0;
+
+	if (!td) {
+#ifndef WIN32
+		td = "/tmp";
+#else
+		if ( !(td=getenv("PVM_TMP")) && !(td=getenv("TEMP")) ) {
+			td = "C:\\TEMP";
+			fprintf(stderr,
+					"Could not get %%PVM_TMP%% or %%TEMP%% ");
+			fprintf(stderr, "- using \"%s\".\n", td);
+			fprintf(stderr, "Please check your installation.\n");
 		}
 #endif
-		pvmbailout(0);
-		exit(1);		/* the other meaning of bail out */
 	}
-	return rd;
+
+	return td;
 }
 
 
@@ -622,7 +826,9 @@ char *
 pvmdsockfile()
 {
 	static char buf[160];
-
+#ifdef WIN32
+	char TDSOCKSPEC[256];
+#endif
 	char hna[128];
 	char *p;
 #ifdef	IMA_CSPP
@@ -630,12 +836,14 @@ pvmdsockfile()
 #endif
 
 #ifdef	SHAREDTMP
+
 	if (gethostname(hna, sizeof(hna)-1) == -1) {
 		pvmlogerror("pvmdsockfile() can't gethostname()\n");
 		return (char*)0;
 	}
 	if (p = CINDEX(hna, '.'))
 		*p = 0;
+
 #ifdef	IMA_CSPP
 	if (scid > 1)
 		(void)sprintf(buf, TDSOCKNAME_CSPP, pvm_useruid, scid, hna);
@@ -644,12 +852,34 @@ pvmdsockfile()
 		(void)sprintf(buf, TDSOCKNAME, pvm_useruid, hna);
 
 #else
+
 #ifdef	IMA_CSPP
 	if (scid > 1)
 		(void)sprintf(buf, TDSOCKNAME_CSPP, pvm_useruid, scid);
 	else
 #endif
+#ifdef WIN32
+
+		p = pvmgettmp();
+
+		if (chdir(p) == -1) {
+			pvmlogprintf(
+				"Could Not Retrieve PVM Temporary Directory \"%s\".\n",
+				p);
+			pvmbailout(0);
+		}
+
+		strcpy(TDSOCKSPEC, p);
+
+		sprintf(buf,"\\pvmd.%s",username);
+		strcat(TDSOCKSPEC,buf);
+
+		sprintf(buf,TDSOCKSPEC);
+
+#else
 		(void)sprintf(buf, TDSOCKNAME, pvm_useruid);
+#endif
+
 #endif
 
 	return buf;
@@ -736,9 +966,12 @@ static char *taglist1[] = {
 	"TM_SCHED",
 	"TM_TASKER",
 	"TM_HOSTER",
+	"TM_TRACER",
 	"TM_HOSTSYNC",
 	"TM_SETOPT",
 	"TM_GETOPT",
+	"TM_CONTEXT",
+	"TM_SHMCONN",
 };
 
 static char *taglist2[] = {
@@ -757,6 +990,7 @@ static char *taglist2[] = {
 	"DM_MCA",
 	"DM_NOTIFY", "DM_NOTIFYACK",
 	"DM_DB", "DM_DBACK",
+	"DM_RESET", "DM_RESETACK",
 	"DM_HTDEL",
 	"DM_HOSTSYNC", "DM_HOSTSYNCACK",
 };
@@ -767,8 +1001,13 @@ static char *taglist3[] = {
 	"TC_TASKEXIT",
 	"TC_NOOP",
 	"TC_OUTPUT",
+	"TC_SETTRACE",
 	"TC_SETTMASK",
+	"TC_SETTRCBUF",
+	"TC_SETTRCOPT",
 	"TC_SHMAT",
+	"TC_OUTPUTX",
+	"TC_SIBLINGS",
 };
 
 static char *taglist4[] = {
@@ -795,9 +1034,9 @@ struct nametag {
 };
 
 static struct nametag nametaglist[] = {
-	{ TM_CONNECT, TM_GETOPT, taglist1 },
+	{ TM_CONNECT, TM_SHMCONN, taglist1 },
 	{ DM_ADD, DM_HOSTSYNCACK, taglist2 },
-	{ TC_CONREQ, TC_SHMAT, taglist3 },
+	{ TC_CONREQ, TC_SIBLINGS, taglist3 },
 	{ SM_SPAWN, SM_STTASK, taglist4 },
 };
 
@@ -832,3 +1071,133 @@ pvmnametag(tag, found)
 }
 
 
+/*	debug_flags()
+*
+*	Convert debug mask to flag names
+*/
+
+static char *dflgs[] = {
+	"pkt",	/* 1 */
+	"msg",	/* 2 */
+	"tsk",	/* 4 */
+	"slv",	/* 8 */
+	"hst",	/* 10 */
+	"sel",	/* 20 */
+	"net",	/* 40 */
+	"mpp",	/* 80 */
+	"sch",	/* 100 */
+	"app",	/* 200 */
+	"wai",	/* 400 */
+	"mem",	/* 800 */
+	"sem",	/* 1000 */
+	"lck",	/* 2000 */
+	"rtg",	/* 4000 */
+	"mbx"	/* 8000 */
+};
+
+char *
+debug_flags(mask)
+	int mask;
+{
+	static char buf[64];
+	int bit, i;
+
+	buf[0] = 0;
+	for (bit = 1, i = 0; i < sizeof(dflgs)/sizeof(dflgs[0]); i++, bit *= 2)
+		if (mask & bit) {
+			if (buf[0])
+				strcat(buf, ",");
+			strcat(buf, dflgs[i]);
+		}
+	return buf;
+}
+
+
+/*	print_fdset()
+*
+*	Print an fd_set in human-readable format.
+*/
+
+print_fdset(pad, n, f)
+	char *pad;			/* label at head */
+	int n;				/* max fd + 1 */
+#ifdef FDSETNOTSTRUCT
+	fd_set *f;			/* fd set */
+#else
+	struct fd_set *f;	/* fd set */
+#endif
+{
+	int i;
+	char *s = "";
+
+	pvmlogprintf("%s", pad);
+	for (i = 0; i < n; i++)
+		if (FD_ISSET(i, f)) {
+			pvmlogprintf("%s%d", s, i);
+			s = ",";
+		}
+	pvmlogprintf("\n");
+	return 0;
+}
+
+
+#ifdef I860_NODE
+int
+gettimeofday(tp, tzp)
+	struct timeval *tp;
+	struct timezone *tzp;
+{
+	double t, dclock();
+
+	t = dclock();
+	tp->tv_sec = t;
+	tp->tv_usec = (t - tp->tv_sec)*1.0E06;
+	return 0;
+}
+#endif
+
+
+/*	pvmgetclock()
+*
+*	Use this whenever we need a clock value in seconds and microseconds.
+*	Return values shouldn't be compared to those from gettimeofday,
+*	because it doesn't necessarily return the actual time of day.
+*/
+
+int
+pvmgetclock(tm)
+	struct timeval *tm;
+{
+#if defined(IMA_PGON)
+	double dclock();
+	double t;
+
+	t = dclock();
+	tm->tv_sec = t;
+	tm->tv_usec = (t - tm->tv_sec)*1.0E06;
+#else
+	return gettimeofday(tm, (struct timezone*)0);
+#endif
+}
+
+
+/*  pvmsleep()
+*
+*   A consistent way to invoke sleep(), from:
+*     Unix / Windows, C / Fortran
+*/
+
+int
+pvmsleep( secs )
+int secs;
+{
+	int ret;
+#ifdef WIN32
+	Sleep( secs * 1000 );
+	ret = secs;
+#else
+	ret = (int) sleep( secs );
+#endif
+	return( ret );
+}
+
