@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: pvmdshmem.c,v 1.16 1998/08/13 18:31:15 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -32,8 +36,85 @@
  * Shared-memory MPP interface.
  *
 $Log: pvmdshmem.c,v $
- * Revision 1.21  1996/05/13  20:33:25  manchek
- * added a few checks in mpp_init so that if init fails pvmd doesn't segfault
+Revision 1.16  1998/08/13 18:31:15  pvmsrc
+Altered SUNMP to use test and set operations with semaphores
+		for page locking instead of MUTEX and cond vars.
+	Changes are mainly in pvmshmem.h, with lots of #ifdefs changes.
+	Makefile altered to use the PLOCKFILE to indicate the Page Locking
+		INLINE code used (from SUNMP.conf).
+	Some changes effect AIX MP versions which still use conditional
+		variables and may change to semaphores soon.
+(Spanker=fagg)
+
+ * Revision 1.15  1997/11/04  23:21:43  pvmsrc
+ * Added SYSVSTR stuff.
+ * (Spanker=kohl)
+ *
+ * Revision 1.14  1997/07/02  20:27:32  pvmsrc
+ * 	Fixed startup race on shmem to that a shmem task can get fully
+ * 	configured before getting any messages.
+ * 	This involved adding two states
+ * 	TF_PRESHMCONN and TF_SHM.  TF_PRESHMCONN indicates that messages
+ * 	with MM_PRIO set can be sent to a task, but regular messages are
+ * 	queued. This allows shmem tasks to be completely configured
+ * 	before any messages flow.  When the daemon changes the state from
+ * 	TF_PRESHMCONN to TF_SHMCONN it calls shm_wrt_pkts to write any
+ * 	packets that were queued before task state changed to TF_SHMCONN.
+ *
+ * Revision 1.13  1997/06/27  20:54:41  pvmsrc
+ * Allow forked process to be caught by daemon
+ * 	when its not ignoring SIGCLD.
+ *
+ * Revision 1.12  1997/06/25  22:09:34  pvmsrc
+ * Markus adds his frigging name to the author list of
+ * 	every file he ever looked at...
+ *
+ * Revision 1.11  1997/06/24  20:39:19  pvmsrc
+ * Eliminated unnecessary global externs.
+ *
+ * Revision 1.10  1997/06/16  13:42:10  pvmsrc
+ * Upated forkexec arg list.
+ *
+ * Revision 1.9  1997/06/02  13:50:03  pvmsrc
+ * Added missing #include host.h for waitc.h.
+ *
+ * Revision 1.8  1997/06/02  13:26:43  pvmsrc
+ * Changed mpp_input so that spawned processes can be claimed after
+ * 	exiting.  Previously they were claimed as their shadow t0 died
+ * 	and set a flag in work() that would get caught leading to
+ * 	mpp_dredge() finding them sometime later...
+ *
+ * Revision 1.7  1997/05/21  16:01:53  pvmsrc
+ * Updated ifdefs to include AIX4MP arch type.
+ *
+ * Revision 1.6  1997/05/19  15:03:01  pvmsrc
+ * pvmfrgsiz is now set according to memory page size and msg header
+ * 	lengths instead of just the UDP host-host packet size!
+ * Mesg headers are set correctly in mpp_input / mpp_output to include
+ * 	contexts etc
+ *
+ * Revision 1.5  1997/04/25  19:17:29  pvmsrc
+ * Mashed to make it work within 3.4 new messaging system. Added:
+ * 	-	correct includes
+ * 	-	changed ph_cod's to ph_tag's and pk_cod -> pk_tag
+ * 	-	add duplicate of pvmdunix ppi_kill()
+ *
+ * Revision 1.4  1997/01/28  19:27:23  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.3  1996/10/25  13:58:01  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.2  1996/10/24  21:04:50  pvmsrc
+ * Moved #include of "global.h" down below other headers:
+ * 	- need to have all of the structures / types declared before
+ * 		the globals can be declared...
+ *
+ * Revision 1.1  1996/09/23  23:44:33  pvmsrc
+ * Initial revision
  *
  * Revision 1.20  1995/11/02  16:31:23  manchek
  * skip over stale packets from dead tasks in mpp_input
@@ -123,11 +204,14 @@ $Log: pvmdshmem.c,v $
 #include <sys/sem.h>
 #include <unistd.h>
 #include <stdlib.h>
+#ifdef  SYSVSTR
+#include <string.h>
+#else
+#include <strings.h>
+#endif
 
-#include "global.h"
-#include "ddpro.h"
-#include "tdpro.h"
-#include "protoglarp.h"
+#include <pvm3.h>
+#include <pvmproto.h>
 #include "pvmalloc.h"
 #include "pvmdabuf.h"
 #include "pkt.h"
@@ -135,7 +219,9 @@ $Log: pvmdshmem.c,v $
 #include "listmac.h"
 #include "pvmshmem.h"
 #include "bfunc.h"
-#include "fromlib.h"
+#include "host.h"
+#include "waitc.h"
+#include "global.h"
 
 #ifndef max
 #define max(a,b)    ((a)>(b)?(a):(b))
@@ -148,15 +234,12 @@ $Log: pvmdshmem.c,v $
 char *getenv();
 
 
-extern int debugmask;			/* from pvmd.c */
+extern int pvmdebmask;			/* from pvmd.c */
 extern int pvm_useruid;			/* from pvmd.c */
 extern int pvmmytid;			/* from pvmd.c */
-extern int myndf;				/* from pvmd.c */
-extern int myhostpart;			/* from pvmd.c */
-extern int tidhmask;			/* from pvmd.c */
-extern int ourudpmtu;			/* from pvmd.c */
-extern int myunixpid;			/* from pvmd.c */
-extern struct task *locltasks;	/* from task.c */
+extern int pvmmydsig;			/* from pvmd.c */
+extern int pvmudpmtu;			/* from pvmd.c */
+extern int pvmmyupid;			/* from pvmd.c */
 extern struct peer *peers;		/* from pvmshmem.c */
 
 
@@ -176,14 +259,14 @@ int maxpidtid = 0;				/* size of pid-tid table */
 int shmbufsiz = 0;				/* shared-memory buffer size */
 int myshmbufid = -1;			/* shared-memory msg buffer ID */
 char *pvminbox = 0;				/* incoming message header buffer */
-
+extern int pvmfrgsiz;			/* From pvmd.c frag size */
+								/* included here as we change it in ppi_config*/
 
 /***************
  **  Private  **
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmdshmem.c,v 1.21 1996/05/13 20:33:25 manchek Exp $";
 static char pvmtxt[512];		/* scratch for error log */
 static int inboxsz = 0;			/* size of incoming message buffer */
 static struct pkt *ovfpkts = 0;	/* packets waiting to be delivered */
@@ -221,15 +304,19 @@ static int globid = -1;			/* ID of global shared segment */
 
 
 int
-mpp_setmtu()
+ppi_config(argc, argv)
+	int argc;
+	char **argv;
 {
 	pgsz = sysconf(_SC_PAGESIZE);
-	ourudpmtu = pgsz - PVMPAGEHDR;
+	pvmudpmtu = pgsz - PVMPAGEHDR; /* XXX yuck host to host limited */
+	pvmfrgsiz = pvmudpmtu;	/* host-task limit.. not same as UDP host to host */
+					/* except h-h is not same as it should be, see above */
 	return 0;
 }
 
 
-/*	mpp_init()
+/*	ppi_init()
 *
 *	Create our shared memory segment, initialize stuff, etc.
 *
@@ -237,9 +324,7 @@ mpp_setmtu()
 */
 
 void
-mpp_init(argc, argv)
-	int *argc;
-	char **argv;
+ppi_init()
 {
 	struct pidtidhdr *pvminfo;
 	char *p;
@@ -268,7 +353,7 @@ mpp_init(argc, argv)
 
 	key = pvmshmkey(0);
 	if ((myshmbufid = shmget((key_t)key, shmbufsiz, IPC_CREAT|PERMS)) == -1) {
-		pvmlogperror("mpp_init() can't create msg buffer");
+		pvmlogperror("ppi_init() can't create msg buffer");
 		pvmbailout(0);
 		return;
 	}
@@ -278,15 +363,17 @@ mpp_init(argc, argv)
 	if ((pvminbox = (char *)shmat(myshmbufid, 0, 0)) == (char *)-1L)
 #endif
 	{
-		pvmlogperror("mpp_init() can't attach msg buffer");
+		pvmlogperror("ppi_init() can't attach msg buffer");
 		shmctl(myshmbufid, IPC_RMID, (struct shmid_ds *)0);
+		myshmbufid = -1;
+		pvmbailout(0);
 		return;
 	}
 
 	infopage = pvminbox + INBOXPAGE*pgsz;
 	outmsgbuf = infopage + pgsz;
 	if (!(outbufsz = (shmbufsiz - INBOXPAGE*pgsz - pgsz)/pvmpgsz)) {
-		pvmlogerror("mpp_init() SHMBUFSIZE too small!");
+		pvmlogerror("ppi_init() SHMBUFSIZE too small!");
 		shmctl(myshmbufid, IPC_RMID, (struct shmid_ds *)0);
 		myshmbufid = -1;
 		pvmbailout(0);
@@ -301,10 +388,10 @@ mpp_init(argc, argv)
 	((struct shmpghdr *)infopage)->pg_ref = 0;
 	pvminfo = (struct pidtidhdr *)(infopage + PVMPAGEHDR);
 	pvminfo->i_proto = TDPROTOCOL;
-	pvminfo->i_ndf = myndf;
+	pvminfo->i_dsig = pvmmydsig;
 	pvminfo->i_next = 0;
 	pvminfo->i_bufsiz = shmbufsiz;
-	pvminfo->i_dpid = myunixpid;
+	pvminfo->i_dpid = pvmmyupid;
 	pidtids = (struct pidtid *)(pvminfo + 1);
 	maxpidtid = (pgsz - sizeof(struct pidtidhdr) - PVMPAGEHDR)/sizeof(struct pidtid);
 	BZERO((char *)pidtids, sizeof(struct pidtid)*maxpidtid);
@@ -318,6 +405,7 @@ mpp_init(argc, argv)
 	peer_init();
 
 	ovfpkts = pk_new(0);
+	ovfpkts -> pk_link = ovfpkts->pk_rlink = ovfpkts;
 
 /*
 #if defined(SUN4SOL2)
@@ -344,7 +432,7 @@ mpp_free(tid)
 	struct task *tp;
 	int i;
 
-	if (debugmask & (PDMTASK|PDMNODE)) {
+	if (pvmdebmask & (PDMTASK|PDMNODE)) {
 		sprintf(pvmtxt, "mpp_free() t%x\n", tid);
 		pvmlogerror(pvmtxt);
 	}
@@ -407,12 +495,12 @@ mpp_conn(tp)
 	pidtids[idx].pt_key = 0;
 	pidtids[idx].pt_cond = 0;
 	PAGEUNLOCK(&((struct shmpghdr *)infopage)->pg_lock);
-	if (debugmask & (PDMTASK|PDMMEM)) {
+	if (pvmdebmask & (PDMTASK|PDMMEM)) {
 		sprintf(pvmtxt, "mpp_conn() assigning pidtid[%d] to t%x\n",
 				idx, tp->t_tid);
 		pvmlogerror(pvmtxt);
 	}
-	if (debugmask & PDMTASK) {
+	if (pvmdebmask & PDMTASK) {
 		sprintf(pvmtxt, "mpp_conn() new task t%x\n", tp->t_tid);
 		pvmlogerror(pvmtxt);
 	}
@@ -420,44 +508,6 @@ mpp_conn(tp)
 }
 
 
-/*	mpp_load()
-*
-*	Fork and exec new tasks.  Give them pidtid table entries.
-*/
-
-int
-mpp_load(flags, name, av, count, tids, ptid, nenv, envp)
-	int flags;				/* exec options */
-	char *name;				/* executable */
-	char **av;				/* arg list (argv[-1] must be there) */
-	int count;				/* how many */
-	int tids[];				/* array to store new tids */
-	int ptid;				/* parent task ID */
-	int nenv;				/* num of envars */
-	char **envp;			/* envars */
-{
-	int i;
-	int err = 0;
-	struct task *tp;
-	int cc;
-
-	for (i = 0; i < count; i++) {
-		if (err) {
-			tids[i] = err;
-		} else {
-			if (err = forkexec(flags, av[0], av, nenv, envp, &tp)) {
-				tids[i] = err;
-			} else {
-				tp->t_ptid = ptid;
-				mpp_conn(tp);	/* XXX this can fail, dunno how to clean up */
-				tids[i] = tp->t_tid;
-			}
-		}
-	}
-	return 0;
-}
-
-
 void
 mpp_input()
 {
@@ -500,7 +550,7 @@ mpp_input()
 				task_setpid(tp, src);
 				tp->t_flag &= ~TF_FORKD;
 			}
-			tp->t_flag |= TF_CONN;
+			tp->t_flag |= TF_CONN | TF_SHM;  
 			continue;
 		}
 
@@ -533,21 +583,24 @@ mpp_input()
 			pp = pk_new(0);
 			pp->pk_dat = cp;
 			pp->pk_buf = buf;
-			pp->pk_max = ourudpmtu;
+			pp->pk_max = pvmudpmtu;
 		}
 */
 		pp->pk_len = len;
 		pp->pk_src = src;
 		pp->pk_dst = dst;
 		pp->pk_flag = inmsgs[next].ph_flag;
-		pp->pk_cod = inmsgs[next].ph_cod;
+		pp->pk_tag = inmsgs[next].ph_tag;
+		pp->pk_ctx = inmsgs[next].ph_ctx;
 		pp->pk_enc = inmsgs[next].ph_enc;
 		pp->pk_wid = inmsgs[next].ph_wid;
 		pp->pk_crc = inmsgs[next].ph_crc;
 
 		if (tp = task_find(src)) {
 			loclinpkt(tp, pp);
-			if ((tp->t_flag & TF_CLOSE) && !(tp->t_flag & TF_FORKD)) {
+
+			/* CL tasks caught here. forked ones caught by SIGCHLD/SIGCLD */
+			if ((tp->t_flag & TF_CLOSE) && !(tp->t_flag & TF_FORKD)) { 
 				task_cleanup(tp);
 				task_free(tp);
 			}
@@ -618,7 +671,7 @@ mpp_output(tp, pp)
 		dmsgs = (struct shmpkhdr *)(dboxp + 1);
 		PAGELOCK(&dboxp->mb_lock);
 		if ((next = (dboxp->mb_last + 1) % inboxsz) != dboxp->mb_read) {
-			if (debugmask & PDMPACKET) {
+			if (pvmdebmask & PDMPACKET) {
 				sprintf(pvmtxt,
 					"mpp_output() src t%x dst t%x ff %x len %d\n",
 					pp->pk_src, pp->pk_dst, pp->pk_flag & (FFSOM|FFEOM),
@@ -631,7 +684,8 @@ mpp_output(tp, pp)
 			dmsgs[next].ph_dat = loc;
 			dmsgs[next].ph_len = pp->pk_len;
 			dmsgs[next].ph_flag = pp->pk_flag & (FFSOM|FFEOM);
-			dmsgs[next].ph_cod = pp->pk_cod;
+			dmsgs[next].ph_ctx = pp->pk_ctx;
+			dmsgs[next].ph_tag = pp->pk_tag;
 			dmsgs[next].ph_enc = pp->pk_enc;
 			dmsgs[next].ph_wid = pp->pk_wid;
 			dmsgs[next].ph_crc = pp->pk_crc;
@@ -639,16 +693,19 @@ mpp_output(tp, pp)
 			dboxp->mb_last = next;
 
 			if (dboxp->mb_sleep) {
-#if defined(IMA_SUNMP) || defined(IMA_RS6KMP)
+
+/* #if defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) */
+#ifdef PVMUSEMUTEX
 #ifdef	IMA_SUNMP
 				cond_signal(&dboxp->mb_cond);
 #endif
-#ifdef	IMA_RS6KMP
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
 				pthread_cond_signal(&dboxp->mb_cond);
 #endif
 #else
 				peer_wake(pe);
-#endif
+
+#endif	/* PVMUSEMUTEX */
 				dboxp->mb_sleep = 0;
 			}
 
@@ -687,7 +744,10 @@ mpp_probe()
 	int ntids, i;
 
 	tosend = ovfpkts;
+
 	ovfpkts = pk_new(0);
+	ovfpkts -> pk_link = ovfpkts -> pk_rlink = ovfpkts;
+
 	while ((pp = tosend->pk_link) != tosend) {
 		LISTDELETE(pp, pk_link, pk_rlink);
 		dst = pp->pk_dst;
@@ -713,6 +773,7 @@ mpp_probe()
 			pk_free(pp);
 		}
 	}
+
 	pk_free(tosend);
 
 	ntids = min(maxpidtid, ((struct shmpghdr *)infopage)->pg_ref);
@@ -728,8 +789,9 @@ mpp_probe()
 
 /*	mpp_cleanup()
 *
-*	We're bailing out.  Remove our shared segment and as many segments
-*	and semaphores as we can for our tasks.
+*	We're bailing out.  All hands on deck. Remove our shared segment and 
+*   as many segments and semaphores as we can for our tasks.
+*
 */
 
 void
@@ -882,3 +944,108 @@ mpp_dredge()
 }
 
 
+/*	ppi_load()
+*
+*	Fork and exec new tasks.  Give them pidtid table entries.
+*/
+
+int
+ppi_load(wxp)
+	struct waitc_spawn *wxp;
+{
+	int i;
+	struct task *tp;
+	int err = 0;
+
+	for (i = 0; i < wxp->w_veclen; i++) {
+		if (err) {
+			wxp->w_vec[i] = err;
+
+		} else {
+			if (err = forkexec(wxp->w_flags, wxp->w_argv[0], wxp->w_argv,
+					wxp->w_nenv, wxp->w_env, 
+					(wxp->w_instance+i), wxp->w_hosttotal, wxp->w_outof, &tp)) {
+				wxp->w_vec[i] = err;
+
+			} else {
+				tp->t_ptid = wxp->w_ptid;
+				tp->t_outtid = wxp->w_outtid;
+				tp->t_outctx = wxp->w_outctx;
+				tp->t_outtag = wxp->w_outtag;
+				tp->t_trctid = wxp->w_trctid;
+				tp->t_trcctx = wxp->w_trcctx;
+				tp->t_trctag = wxp->w_trctag;
+				tp->t_sched = wxp->w_sched;
+				mpp_conn(tp);	/* XXX this can fail, dunno how to clean up */
+				wxp->w_vec[i] = tp->t_tid;
+			}
+		}
+	}
+	return 0;
+}
+
+
+/* XXX this is just a basic copy of the pvmdunix code so we can at least compile */
+int
+ppi_kill(tp, signum)
+    struct task *tp;
+    int signum;
+{
+    if (tp->t_pid)
+        (void)kill(tp->t_pid, signum);
+    else
+        pvmlogprintf("ppi_kill() signal for t%x scrapped (pid = 0)\n",
+                tp->t_tid);
+    return 0;
+}
+
+
+/* write any packets waiting the tp -> t_txq, send queue, these may 
+have been stacked up before the task state changed to  TF_SHMCONN
+This is currently called by tm_getopt when the state change is
+marked from TF_PRESHMCONN to TF_SHMCONN
+*/
+int
+shm_wrt_pkts(tp)
+struct task * tp;
+{
+struct pkt *pp, *pp2, *pptmp;
+struct pkt tmplist;
+
+	
+	if (tp -> t_txq)
+	{
+		pptmp = &tmplist;
+		pptmp -> pk_link = pptmp->pk_rlink = pptmp;
+
+		/* Take packets off of the t_txq */
+		pp = tp -> t_txq -> pk_link;
+		while (pp != tp -> t_txq)
+		{
+			pp2 = pp;
+			pp = pp -> pk_link;
+			LISTDELETE(pp2, pk_link, pk_rlink);
+			LISTPUTBEFORE(pptmp, pp2, pk_link, pk_rlink);
+		}
+
+		pp = pptmp -> pk_link;
+
+		while (pp != pptmp)
+		{
+			pp2 = pp;
+			pp = pp -> pk_link;
+			LISTDELETE(pp2, pk_link, pk_rlink);
+			if (tp -> t_flag & TF_SHMCONN)
+			{
+				if ( pvmdebmask & PDMPACKET)
+					pvmlogprintf("shm_wrt_pkts for %x \n", tp -> t_tid);
+				mpp_output(tp, pp2);
+			}
+			else
+				break;
+		}	
+	}
+		
+	return 0;
+}
+	
