@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: pvmshmem.c,v 1.9 1998/08/13 18:31:12 pvmsrc Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -32,6 +36,45 @@
  *  Shared-memory stuff.
  *
 $Log: pvmshmem.c,v $
+Revision 1.9  1998/08/13 18:31:12  pvmsrc
+Altered SUNMP to use test and set operations with semaphores
+		for page locking instead of MUTEX and cond vars.
+	Changes are mainly in pvmshmem.h, with lots of #ifdefs changes.
+	Makefile altered to use the PLOCKFILE to indicate the Page Locking
+		INLINE code used (from SUNMP.conf).
+	Some changes effect AIX MP versions which still use conditional
+		variables and may change to semaphores soon.
+(Spanker=fagg)
+
+ * Revision 1.8  1998/03/05  16:31:31  pvmsrc
+ * Hack for CSPP - _touch() wasn't typed before use...  :-Q
+ * (Spanker=kohl)
+ *
+ * Revision 1.7  1997/07/08  20:18:07  pvmsrc
+ * Better logging in peer_conn()
+ *
+ * Revision 1.6  1997/06/25  22:11:10  pvmsrc
+ * Markus adds his frigging name to the author list of
+ * 	every file he ever looked at...
+ *
+ * Revision 1.5  1997/05/21  16:01:55  pvmsrc
+ * Updated ifdefs to include AIX4MP arch type.
+ *
+ * Revision 1.4  1997/04/25  19:18:14  pvmsrc
+ * Corrected includes.
+ *
+ * Revision 1.3  1997/01/28  19:27:30  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.2  1996/10/25  14:19:06  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.1  1996/09/23  23:44:40  pvmsrc
+ * Initial revision
+ *
  * Revision 1.21  1995/11/02  16:33:04  manchek
  * peer_conn returns -1 if peer has disconnected
  *
@@ -121,18 +164,20 @@ $Log: pvmshmem.c,v $
 #endif
 #include <errno.h>
 
-#include "protoglarp.h"
-#include "ddpro.h"
+#include <pvm3.h>
+#include <pvmproto.h>
 #include "pvmalloc.h"
 #include "listmac.h"
 #include "pvmshmem.h"
 #include "bfunc.h"
 
+#include "global.h"
+
 #ifndef min
 #define min(a,b)	((a)<(b)?(a):(b))
 #endif
 
-extern int debugmask;				/* from pvmd.c or lpvmshmem.c */
+extern int pvmdebmask;				/* from pvmd.c or lpvmshmem.c */
 extern int pvmmytid;				/* from pvmd.c or lpvmshmem.c */
 extern int pvmpgsz;					/* from pvmdshmem.c or lpvmshmem.c */
 extern int pgsz;					/* from pvmdshmem.c or lpvmshmem.c */
@@ -162,10 +207,13 @@ int bufpageused = 0;				/* number of dirty pages in msg buf */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmshmem.c,v 1.21 1995/11/02 16:33:04 manchek Exp $";
 static char pvmtxt[512];			/* scratch for error log */
 static int nxtpage = 0;				/* next free page in outgoing msg buf */
 
+#ifdef IMA_CSPP
+static void _touch();
+#endif
+
 
 /*******************************************************
  **  Shared-memory data buffer manipulation routines  **
@@ -185,10 +233,10 @@ msgbufinit(buf)
 
 	end = buf + shmbufsiz;
 #ifndef IMA_KSR1
-#ifdef IMA_SUNMP
+#if defined(IMA_SUNMP) && defined(PVMUSEMUTEX)
 	cond_init(&((struct msgboxhdr *)buf)->mb_cond, USYNC_PROCESS, 0);
 #endif
-#ifdef IMA_RS6KMP
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
 	pthread_cond_init(&((struct msgboxhdr *)buf)->mb_cond, NULL);
 #endif
 	PAGEINITLOCK(&((struct msgboxhdr *)buf)->mb_lock);
@@ -205,7 +253,7 @@ msgbufinit(buf)
 		((struct shmpghdr *)p)->pg_ref = 0;
 	}
 
-	if (debugmask & PDMNODE) {
+	if (pvmdebmask & PDMNODE) {
 		sprintf(pvmtxt, "msgbufinit() outmsgbuf=0x%x\n", outmsgbuf);
 		pvmlogerror(pvmtxt);
 		sprintf(pvmtxt, "msgbufinit() last buffer=0x%x\n", p - pvmpgsz);
@@ -360,10 +408,22 @@ peer_conn(tid, new_connection)
 		*new_connection = 0;
 	for (pp = peers->p_link; pp != peers; pp = pp->p_link) {
 		if (pp->p_tid == tid) {		/* already connected */
-			if (pp->p_exited || !pp->p_buf)
+			if (pp->p_exited || !pp->p_buf) {
+				if (pvmdebmask & PDMMEM) {
+					sprintf(pvmtxt, "peer_conn() t%x has exited?\n", tid);
+				 	pvmlogerror(pvmtxt);
+				 }
 				return ((struct peer *)-1L);
-			else
+				}
+			else {
+				if (pvmdebmask & PDMMEM) {
+					sprintf(pvmtxt, "peer_conn() to t%x got pp addr %lx\n", 
+							tid, pp);
+				 	pvmlogerror(pvmtxt);
+				 }
 				return pp;
+
+				}
 		}
 	/*
 	* test p_buf in case we're in the process of destroying this peer struct
@@ -407,9 +467,15 @@ peer_conn(tid, new_connection)
 				/* must be ST_SHMEM or ST_FINISH */
 				break;
 			}
-		if (i == ntids)
+		if (i == ntids) 
+			{
+			if (pvmdebmask & PDMMEM) {
+				sprintf(pvmtxt, "peer_conn() could not find t%x in daemon list\n", tid);
+			 pvmlogerror(pvmtxt);
+			 }
 			return (struct peer *)-1L;
 		/* PAGEUNLOCK(pvminfo); */
+		}
 
 		key = pidtids[i].pt_key;
 		if ((bufid = shmget((key_t)key, shmbufsiz, 0)) == -1) {
@@ -418,7 +484,8 @@ peer_conn(tid, new_connection)
 			return (struct peer *)0;
 		}
 		while ((p = (char *)shmat(bufid, 0, 0)) == (char *)-1L) {
-			sprintf(pvmtxt, "peer_conn() shmat to t%x", tid);
+			sprintf(pvmtxt, "peer_conn() shmat to t%x using bufid %d\n", 
+					tid, bufid);
 			pvmlogperror(pvmtxt);
 			if (errno == EMFILE && (pp = peers->p_rlink) != peers) {
 				shmdt(pp->p_buf);
@@ -481,7 +548,7 @@ peer_wake(pp)
 		sprintf(pvmtxt, "peer_wake() semop t%x\n", pp->p_tid);
 		pvmlogperror(pvmtxt);
 
-	} else if (debugmask & PDMSEM) {
+	} else if (pvmdebmask & PDMSEM) {
 		sprintf(pvmtxt,"peer_wake() sem %d\n", pp->p_semid);
 		pvmlogerror(pvmtxt);
 	}
@@ -608,7 +675,7 @@ pvmshmkey(pid)
 		if (!(pid &= SHMIDPIDPART))
 			pid = 1;
 		k = pvmd_key | pid;
-		if (debugmask & (PDMMEM | PDMSEM)) {
+		if (pvmdebmask & (PDMMEM | PDMSEM)) {
 			sprintf(pvmtxt, "pvmshmkey(): shm/sem key is 0x%x\n", k);
 			pvmlogerror(pvmtxt);
 		}
@@ -616,7 +683,7 @@ pvmshmkey(pid)
 	} else {
 		k = pvmd_key;
 
-		if (debugmask & (PDMMEM | PDMSEM)) {
+		if (pvmdebmask & (PDMMEM | PDMSEM)) {
 			sprintf(pvmtxt, "pvmshmkey(): pvmd shm/sem key is 0x%x\n", k);
 			pvmlogerror(pvmtxt);
 		}
@@ -641,7 +708,7 @@ nextpvmshmkey(key)
 	if (!pid)
 		pid = 1;
 	pid |= pvmshmkey(0);
-	if (debugmask & (PDMMEM | PDMSEM)) {
+	if (pvmdebmask & (PDMMEM | PDMSEM)) {
 		sprintf(pvmtxt, "nextpvmshmkey() shm/sem key is 0x%x\n", pid);
 		pvmlogerror(pvmtxt);
 	}
@@ -657,7 +724,7 @@ init_lock(vp, f, l)
 	char *f;
 	int l;
 {
-	if (debugmask & PDMLOCK) {
+	if (pvmdebmask & PDMLOCK) {
 		sprintf(pvmtxt,"init_lock(): init %#x from %s line %d\n", vp,f,l);
 		pvmlogerror(pvmtxt);
 	}
@@ -674,7 +741,7 @@ acquire_lock(vp, f, l)
 	int * volatile ip = (int *)vp;
 	int i=0, j=0;
 
-	if (debugmask & PDMLOCK) {
+	if (pvmdebmask & PDMLOCK) {
 		sprintf(pvmtxt,"acquire_lock(): polling %#x from %s line %d\n", ip,f,l);
 		pvmlogerror(pvmtxt);
 	}
@@ -686,7 +753,7 @@ acquire_lock(vp, f, l)
 		j++;
 	}
 
-	if (debugmask & PDMLOCK) {
+	if (pvmdebmask & PDMLOCK) {
 		if (i)
 			sprintf(pvmtxt,"acquire_lock(): acquired %#x after %d/%d polls\n",
 				ip, i, j);
@@ -703,7 +770,7 @@ release_lock(vp,f,l)
 	char *f;
 	int l;
 {
-	if (debugmask & PDMLOCK) {
+	if (pvmdebmask & PDMLOCK) {
 		sprintf(pvmtxt,"release_lock(): releasing %#x from %s line %d\n", vp,f,l);
 		pvmlogerror(pvmtxt);
 	}
@@ -734,7 +801,7 @@ char *addr;
             char *shm_pg;
             int pgsz = sysconf(_SC_PAGESIZE);
 
-			if(debugmask & PDMMEM) {
+			if(pvmdebmask & PDMMEM) {
 				sprintf(pvmtxt, "Touching block at addr 0x%x (%d bytes)\n",
 					addr, shmbufsiz);
 				pvmlogerror(pvmtxt);
@@ -750,7 +817,7 @@ char *addr;
 
 	default:
 		touch_flg = (getenv("PVMTOUCH") ? TOUCH_PAGES : DONT_TOUCH);
-		if(debugmask & (PDMTASK|PDMMEM)) {
+		if(pvmdebmask & (PDMTASK|PDMMEM)) {
 			sprintf(pvmtxt, "Shared memory touching: %s\n",
 				touch_flg ? "ON" : "OFF" );
 			pvmlogerror(pvmtxt);
@@ -781,7 +848,7 @@ int bufid;
 	for(i = GATEWY_LO_ADDR; i > SHM_LO_ADDR && unmapped; i-=shm_incr_addr) {
 		inbox = (char*)shmat(bufid, (char*) i, 0);
 		unmapped = (inbox == (char*)-1L);
-		if ( unmapped && (debugmask & PDMMEM) ) {
+		if ( unmapped && (pvmdebmask & PDMMEM) ) {
 			sprintf(pvmtxt, "shm_search(): shmat() failed attach at %#x\n", i);
 			pvmlogerror(pvmtxt);
 		}
@@ -790,7 +857,7 @@ int bufid;
 	for(i = GATEWY_HI_ADDR; i < SHM_HI_ADDR && unmapped; i+=shm_incr_addr) {
 		inbox = (char*)shmat(bufid, (char*) i, 0);
 		unmapped = (inbox == (char*)-1L);
-		if ( unmapped && (debugmask & PDMMEM) ) {
+		if ( unmapped && (pvmdebmask & PDMMEM) ) {
 			sprintf(pvmtxt, "shm_search(): shmat() failed attach at %#x\n", i);
 			pvmlogerror(pvmtxt);
 		}
@@ -801,7 +868,7 @@ int bufid;
 		return (char *) -1;
 	}
 
-	if (debugmask & PDMMEM) {
+	if (pvmdebmask & PDMMEM) {
 		sprintf(pvmtxt, "shm_search(): shmat attached to addr %x\n",
 			(unsigned int) inbox);
 		pvmlogerror(pvmtxt);
