@@ -33,16 +33,16 @@
  *  pvmshmem.h
  *
 $Log: pvmshmem.h,v $
- * Revision 1.6  1998/08/13  18:31:11  pvmsrc
- * Altered SUNMP to use test and set operations with semaphores
- * 		for page locking instead of MUTEX and cond vars.
- * 	Changes are mainly in pvmshmem.h, with lots of #ifdefs changes.
- * 	Makefile altered to use the PLOCKFILE to indicate the Page Locking
- * 		INLINE code used (from SUNMP.conf).
- * 	Some changes effect AIX MP versions which still use conditional
- * 		variables and may change to semaphores soon.
- * (Spanker=fagg)
- *
+Revision 1.6  1998/08/13 18:31:11  pvmsrc
+Altered SUNMP to use test and set operations with semaphores
+		for page locking instead of MUTEX and cond vars.
+	Changes are mainly in pvmshmem.h, with lots of #ifdefs changes.
+	Makefile altered to use the PLOCKFILE to indicate the Page Locking
+		INLINE code used (from SUNMP.conf).
+	Some changes effect AIX MP versions which still use conditional
+		variables and may change to semaphores soon.
+(Spanker=fagg)
+
  * Revision 1.5  1997/08/06  22:43:23  pvmsrc
  * Added new SGI6 and SGIMP6 arches.
  *
