@@ -36,12 +36,12 @@ static char rcsid[] =
  *	Entry points for messages from local tasks.
  *
 $Log: tdpro.c,v $
- * Revision 1.27  1998/11/20  20:06:49  pvmsrc
- * Changes so that win32 will compile & build. Also, common
- * Changes so that compiles & builds on NT. Also
- * common source on win32 & unix.
- * (spanker=sscott)
- *
+Revision 1.27  1998/11/20 20:06:49  pvmsrc
+Changes so that win32 will compile & build. Also, common
+Changes so that compiles & builds on NT. Also
+common source on win32 & unix.
+(spanker=sscott)
+
  * Revision 1.26  1998/01/12  21:13:31  pvmsrc
  * Replaced inline constants with new task output op defines.
  * 	- TO_NEW == -2.
