@@ -36,12 +36,12 @@ static char rcsid[] =
  *	Wait context descriptors.
  *
 $Log: waitc.c,v $
- * Revision 1.7  1998/11/20  20:04:19  pvmsrc
- * Changes so that win32 will compile & build. Also, common
- * Changes so that compiles & builds on NT. Also
- * common source on win32 & unix.
- * (Spanker=sscott)
- *
+Revision 1.7  1998/11/20 20:04:19  pvmsrc
+Changes so that win32 will compile & build. Also, common
+Changes so that compiles & builds on NT. Also
+common source on win32 & unix.
+(Spanker=sscott)
+
  * Revision 1.6  1997/06/27  17:32:59  pvmsrc
  * Updated for WIN32 header files & Authors.
  *
